# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from zope.component import getUtility
from zope.intid.interfaces import IIntIds

from Products.Silva.testing import TestRequest, Transaction
from Products.Silva.tests.test_xml_export import SilvaXMLTestCase

from zeam.form import silva as silvaforms
from zeam.component import getWrapper

from ..testing import FunctionalLayer
from ..interfaces import IExternalSourceManager


HTML_CITATION_CODE_SOURCE = u"""
<div>
    <h1>Example</h1>
    <p>some paragraph..</p>
    <div class="external-source default"
         data-silva-name="cs_citation"
         data-silva-settings="field_citation=blahblahblah&amp;field_author=ouam&amp;field_source=wikipedia">
    </div>
</div>
"""
HTML_TOC_CODE_SOURCE = u"""
<div>
    <h1>Document with TOCs</h1>
    <p>A first table of content:</p>
    <div class="external-source default"
         data-silva-name="cs_toc"
         data-silva-settings="field_paths={0}&amp;field_toc_types=Silva+Folder&amp;field_depth=0&amp;field_sort_on=silva&amp;field_order=normal">
    </div>
    <p>Since we were not sure about it, a second table of content:</p>
    <div class="external-source default"
         data-silva-name="cs_toc"
         data-silva-settings="field_paths={0}&amp;field_toc_types=Silva+Folder&amp;field_depth=0&amp;field_sort_on=silva&amp;field_order=normal">
    </div>
</div>
"""

class TOCCodeSourceDocumentExportTestCase(SilvaXMLTestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        with Transaction():
            self.layer.login('author')
            # You have to install the source as manager
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Folder')
            factory = self.root.folder.manage_addProduct['silva.app.document']
            factory.manage_addDocument('example', 'Example')
            version = self.root.folder.example.get_editable()
            html = HTML_TOC_CODE_SOURCE.format(
                getUtility(IIntIds).register(self.root.folder))
            version.body.save(version, TestRequest(), html)

    def test_export_two_tocs_in_document(self):
        exporter = self.assertExportEqual(
            self.root.folder,
            'test_export_source_two_tocs.silvaxml')
        self.assertEqual(exporter.getZexpPaths(), [])
        self.assertEqual(exporter.getAssetPaths(), [])
        self.assertEqual(exporter.getProblems(), [])


class CitationCodeSourceDocumentExportTestCase(SilvaXMLTestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        with Transaction():
            self.layer.login('manager')
            # You have to install the source as manager
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Folder')
            token = self.root.manage_cutObjects(['cs_citation'])
            self.root.folder.manage_pasteObjects(token)
        with Transaction():
            self.layer.login('author')
            # Continue as author
            factory = self.root.folder.manage_addProduct['silva.app.document']
            factory.manage_addDocument('example', 'Example')
            version = self.root.folder.example.get_editable()
            version.body.save(version, TestRequest(), HTML_CITATION_CODE_SOURCE)

    def test_export_code_source_and_document(self):
        """Export a document containing a code source.

        The code source is exported as a ZEXP.
        """
        exporter = self.assertExportEqual(
            self.root.folder,
            'test_export_source.silvaxml')
        self.assertEqual(
            exporter.getZexpPaths(),
            [(('', 'root', 'folder', 'cs_citation'), '1.zexp')])
        self.assertEqual(
            exporter.getAssetPaths(),
            [])
        self.assertEqual(
            exporter.getProblems(),
            [])

    def test_export_document_with_missing_source(self):
        """Export a document containing a missing code source.
        """
        with Transaction():
            # Delete code source to break it.
            self.root.manage_delObjects(['cs_citation'])
            self.root.folder.manage_delObjects(['cs_citation'])

        # Export
        exporter = self.assertExportEqual(
            self.root.folder,
            'test_export_source_missing.silvaxml')
        self.assertEqual(exporter.getZexpPaths(), [])
        self.assertEqual(exporter.getAssetPaths(), [])

        version = self.root.folder.example.get_editable()
        self.assertEqual(
            exporter.getProblems(),
            [(u'Broken source in document while exporting: '
              u'source is no longer installed in the Silva site.',
              version)])


class SourceAssertExportTestCase(SilvaXMLTestCase):
    """Test source asset XML export. We will use the TOC code source
    for the test.
    """

    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        with Transaction():
            self.layer.login('editor')
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Folder')
            factory.manage_addFolder('other', 'Other Folder')
            factory = self.root.folder.manage_addProduct['SilvaExternalSources']
            factory.manage_addSourceAsset('asset', 'A source asset')

    def save_asset(self, target=None):
        with Transaction():
            # Helper to save the parameters in the test source asset.
            version = self.root.folder.asset.get_editable()

            if target is not None:
                target = getUtility(IIntIds).register(target)
            else:
                target = '0'
            request = TestRequest(
                form={'field_paths': str(target),
                      'field_toc_types': "Silva Folder",
                      'field_depth': "0",
                      'field_sort_on': "silva",
                      'field_order': "normal"})
            factory = getWrapper(version, IExternalSourceManager)
            source = factory(request, name='cs_toc')
            marker = source.create()
            version.set_parameters_identifier(source.getId())
        self.assertIs(marker, silvaforms.SUCCESS)
        return version

    def test_export_source_asset(self):
        self.save_asset(self.root.folder)

        exporter = self.assertExportEqual(
            self.root.folder,
            'test_export_source_asset.silvaxml')
        self.assertEqual(exporter.getZexpPaths(), [])
        self.assertEqual(exporter.getAssetPaths(), [])
        self.assertEqual(exporter.getProblems(), [])

    def test_export_source_asset_external_reference(self):
        """The code source has a parameters that will trigger an
        external reference error.
        """
        self.save_asset(self.root.other)

        self.assertExportFail(self.root.folder)

    def test_export_source_asset_external_reference_force(self):
        """The code source has a parameters that have a reference
        outside of the export, but the option external_references is
        set.
        """
        version = self.save_asset(self.root.other)

        exporter = self.assertExportEqual(
            self.root.folder,
            'test_export_source_asset_external.silvaxml',
            options={'external_references': True})
        self.assertEqual(
            exporter.getZexpPaths(),
            [])
        self.assertEqual(
            exporter.getAssetPaths(),
            [])
        self.assertEqual(
            exporter.getProblems(),
            [(u"A reference field 'paths' refers to an content outside of the export (../other).",
              version)])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CitationCodeSourceDocumentExportTestCase))
    suite.addTest(unittest.makeSuite(TOCCodeSourceDocumentExportTestCase))
    suite.addTest(unittest.makeSuite(SourceAssertExportTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from grokcore.chameleon.components import ChameleonPageTemplate
from zeam.component import getComponent

from silva.ui.rest import UIREST
from silva.core.interfaces import ISilvaObject, IVersionedContent
from silva.core.views import views as silvaviews

from ..interfaces import availableSources
from ..interfaces import IExternalSourceManager
from ..errors import SourceError

from Products.Formulator.zeamsupport import IFormulatorField

# Define the CSS class names to use with CKE editor.
STYLES = {'ListField': {'css_class': 'cke_dialog_ui_input_select'},
          'MultiListField': {'css_class': 'cke_dialog_ui_input_select'},
          'StringField': {'css_class': 'cke_dialog_ui_input_text'},
          'IntegerField': {'css_class': 'cke_dialog_ui_input_text'},
          'EmailField': {'css_class': 'cke_dialog_ui_input_text'},
          'PatternField': {'css_class': 'cke_dialog_ui_input_text'},
          'TextAreaField': {'css_class': 'cke_dialog_ui_input_textarea'},
          'RawTextAreaField': {'css_class': 'cke_dialog_ui_input_textarea'},
          'ListTextAreaField': {'css_class': 'cke_dialog_ui_input_textarea'},
          'LinesField': {'css_class': 'cke_dialog_ui_input_textarea'},
          'EmailLinesField': {'css_class': 'cke_dialog_ui_input_textarea'},
          'PasswordField': {'css_class': 'cke_dialog_ui_input_password'}}


class ListAvailableSources(UIREST):
    """List all available sources.
    """
    grok.context(ISilvaObject)
    grok.name('Products.SilvaExternalSources.source.availables')

    def GET(self):
        sources = []
        for identifier, source in availableSources(self.context):
            sources.append({'identifier': identifier,
                            'title': source.get_title()})
        return self.json_response(sources)


class SourceAPI(UIREST):
    grok.context(ISilvaObject)
    grok.baseclass()

    def get_document(self):
        """Return the document version to which the External source is
        associated.
        """
        if IVersionedContent.providedBy(self.context):
            version = self.context.get_editable()
            if version is not None:
                return version
            version = self.context.get_viewable()
            if version is not None:
                return version
        return self.context

    def get_source(self):
        """Return the External Source and form associated with the
        given request.
        """
        document = self.get_document()
        source = getComponent(document, IExternalSourceManager)(document)(
            self.request,
            instance=self.request.get('source_instance'),
            name=self.request.get('source_name'))
        source.ignoreContent = 'source_name' in self.request.form
        source.ignoreRequest = 'source_inline' not in self.request.form
        return source


class PreviewSource(SourceAPI):
    grok.name('Products.SilvaExternalSources.source.preview')

    def POST(self):
        try:
            source = self.get_source()
            html = source.render(preview=True).strip()
            if not html:
                # XXX Should support translation here (i.e. use a template)
                return """<div><h3>%s</h3>
<p>The source didn't produce any content.</p></div>""" % source.label
            return html
        except SourceError as error:
            return silvaviews.render(error, self.request, preview=True)


class SourceParameters(SourceAPI):
    """Return a form to enter and validate source paramters.
    """
    grok.name('Products.SilvaExternalSources.source.parameters')

    template = ChameleonPageTemplate(filename="templates/parameters.cpt")

    def default_namespace(self):
        return {'rest': self,
                'context': self.context,
                'request': self.request}

    def namespace(self):
        return {}

    def POST(self):
        self.fields = []
        try:
            source = self.get_source()
        except SourceError:
            self.request.response.setStatus(500)
            return ''
        else:
            # Apply formulator styles. No choice here.
            for field in source.fields:
                if IFormulatorField.providedBy(field):
                    if field.meta_type in STYLES:
                        field.customize(STYLES[field.meta_type])

            for widget in source.fieldWidgets(
                    ignoreRequest=False, ignoreContent=False):
                label_class = ['cke_dialog_ui_labeled_label']
                if widget.required:
                    label_class.append('cke_required')
                self.fields.append(
                    {'identifier': widget.htmlId(),
                     'label_class': ' '.join(label_class),
                     'title': widget.title,
                     'description': widget.description,
                     'widget': widget.render()})
            return self.json_response({
                'title': source.label,
                'parameters': self.template.render(self)})


class SourceParametersValidator(SourceAPI):
    """Validate source parameters.
    """
    grok.name('Products.SilvaExternalSources.source.validate')

    def POST(self):
        source = self.get_source()
        if source is not None:
            values, errors = source.extractData(source.fields)
            if errors:
                return self.json_response(
                    {'success': False,
                     'messages': [{'identifier': error.identifier,
                                   'message': error.title}
                                  for error in errors]})
        return self.json_response({'success': True})



# -*- coding: utf-8 -*-
# Copyright (c) 2013  Infrae. All rights reserved.
# See also LICENSE.txt

import unittest
from lxml.etree import tostring, Element

from ..silvaxml.treehandler import ElementTreeContentHandler

NS_TEST1 = 'http://test.com/1'
NS_TEST2 = 'http://test.com/2'


class TreeHandlerTestCase(unittest.TestCase):

    def test_simple_node(self):
        handler = ElementTreeContentHandler()
        handler.startElement('test', {'id': 'value'})
        handler.characters('This is the content of test')
        handler.endElement('test')
        self.assertEqual(
            tostring(handler.etree),
            '<test id="value">This is the content of test</test>')

    def test_simple_node_with_default_namespace(self):
        handler = ElementTreeContentHandler()
        handler.startPrefixMapping(None, NS_TEST1)
        handler.startElement('test', {'id': 'value'})
        handler.characters('Content')
        handler.endElement('test')
        handler.endPrefixMapping(None)
        self.assertEqual(
            tostring(handler.etree),
            '<test xmlns="http://test.com/1" id="value">Content</test>')

    def test_two_node_with_default_namespace(self):
        handler = ElementTreeContentHandler()
        handler.startElement('container')
        handler.startPrefixMapping(None, NS_TEST1)
        handler.startElement('test', {'id': 'value'})
        handler.characters('Content')
        handler.endElement('test')
        handler.endPrefixMapping(None)
        handler.startPrefixMapping(None, NS_TEST1)
        handler.startElement('test', {'id': 'value'})
        handler.characters('Test')
        handler.endElement('test')
        handler.endPrefixMapping(None)
        handler.endElement('container')
        self.assertEqual(
            tostring(handler.etree),
            '<container>'
            '<test xmlns="http://test.com/1" id="value">Content</test>'
            '<test xmlns="http://test.com/1" id="value">Test</test>'
            '</container>')

    def test_two_node_with_default_namespace_on_root(self):
        root = Element('existing')
        handler = ElementTreeContentHandler(root=root)
        handler.startElement('container')
        handler.startPrefixMapping(None, NS_TEST1)
        handler.startElement('test', {'id': 'value'})
        handler.characters('Content')
        handler.endElement('test')
        handler.endPrefixMapping(None)
        handler.startPrefixMapping(None, NS_TEST1)
        handler.startElement('test', {'id': 'value'})
        handler.characters('Test')
        handler.endElement('test')
        handler.endPrefixMapping(None)
        handler.endElement('container')
        self.assertEqual(
            tostring(root),
            '<existing><container>'
            '<test xmlns="http://test.com/1" id="value">Content</test>'
            '<test xmlns="http://test.com/1" id="value">Test</test>'
            '</container></existing>')

    def test_two_node_with_two_default_namespace(self):
        handler = ElementTreeContentHandler()
        handler.startPrefixMapping(None, NS_TEST1)
        handler.startElement('root')
        for i in range(0, 2):
            handler.startElement('container')
            handler.startPrefixMapping(None, NS_TEST2)
            handler.startElement('test', {'id': 'value'})
            handler.characters('Content')
            handler.endElement('test')
            handler.endPrefixMapping(None)
            handler.startPrefixMapping(None, NS_TEST2)
            handler.startElement('test', {'id': 'value'})
            handler.characters('Test')
            handler.endElement('test')
            handler.endPrefixMapping(None)
            handler.endElement('container')
        handler.endElement('root')
        self.assertEqual(
            tostring(handler.etree),
            '<root xmlns="http://test.com/1">'
            '<container>'
            '<test xmlns="http://test.com/2" id="value">Content</test>'
            '<test xmlns="http://test.com/2" id="value">Test</test>'
            '</container>'
            '<container>'
            '<test xmlns="http://test.com/2" id="value">Content</test>'
            '<test xmlns="http://test.com/2" id="value">Test</test>'
            '</container>'
            '</root>')


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TreeHandlerTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from AccessControl.security import checkPermission

from five import grok
from zope.i18n import translate
from silva.core.views import views as silvaviews
from silva.core.views.interfaces import IPreviewLayer
from silva.translations import translate as _


class SourceError(ValueError):
    """Error related with the handling of source objects.
    """

    def message(self):
        return _(u"External Source errored during the processing.")

    def __unicode__(self):
        return translate(self.message())

    def __str__(self):
        return str(self.__unicode__())


class SourceErrorView(silvaviews.Render):
    grok.context(SourceError)

    def render(self, preview=None):
        if preview is None:
            preview = IPreviewLayer.providedBy(self.request)
        if preview:
            return u"".join(
                ("<p>",
                 translate(self.context.message(), context=self.request),
                 "</p>"))
        return u""


class SourceRenderingError(SourceError):
    """A Python error happened during the rendering of the code
    source.
    """

    def __init__(self, manager, info):
        super(SourceRenderingError, self).__init__(manager, info)
        self.identifier = manager.getSourceId()
        self.manager = manager
        self.info = info


class SourceRenderingErrorView(silvaviews.Render):
    grok.context(SourceRenderingError)

    def update(self, preview=None):
        if preview is None:
            preview = IPreviewLayer.providedBy(self.request)
        self.preview = preview
        self.title = self.context.manager.label
        self.info = self.context.info
        self.manage = checkPermission(
            'silva.ManageSilvaSettings', self.context.manager.context)


class SourcePreviewError(SourceError):

    def __init__(self, manager):
        super(SourcePreviewError, self).__init__(manager)
        self.identifier = manager.getSourceId()
        self.manager = manager


class SourcePreviewErrorView(silvaviews.Render):
    grok.context(SourcePreviewError)

    def update(self, preview=None):
        self.title = self.context.manager.label
        self.editable = self.context.manager.editable()
        self.widgets = self.context.manager.fieldWidgets(display=True)


class SourceMissingError(SourceError):
    """The source is not available.
    """

    def __init__(self, identifier):
        super(SourceMissingError, self).__init__(identifier)
        self.identifier = identifier

    def message(self):
        return _(u"External Source ${identifier} is not available.",
                 mapping=dict(identifier=self.identifier))


class ParametersError(SourceError):
    """The parameters are broken.
    """

    def message(self):
        return _("Error while validating the External Source parameters.")


class ParametersMissingError(ParametersError):
    """The parameters are missing.
    """

    def message(self):
        return _("External Source parameters are missing.")





# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import logging

from five import grok
from zope.interface import Interface

from silva.core.editor.transform.base import TransformationFilter
from silva.core.editor.transform.interfaces import ISilvaXMLExportFilter
from silva.core.interfaces import IVersion, ISilvaXMLProducer
from silva.core.xml import producers, NS_SILVA_URI
from zeam.component import getWrapper
from zeam.form.silva.interfaces import IXMLFormSerialization

from . import NS_SOURCE_URI
from ..interfaces import IExternalSourceManager
from ..interfaces import ISourceAsset, ISourceAssetVersion
from ..errors import SourceError
from .treehandler import ElementTreeContentHandler


logger = logging.getLogger('silva.core.xml')


class FieldProducer(ElementTreeContentHandler):

    def __init__(self, context, handler, **kwargs):
        ElementTreeContentHandler.__init__(self, **kwargs)
        self.context = context
        self.__handler = handler

    def getHandler(self):
        return self.__handler


class ExternalSourceExportFilter(TransformationFilter):
    grok.adapts(IVersion, ISilvaXMLProducer)
    grok.provides(ISilvaXMLExportFilter)

    def __init__(self, context, handler):
        self.context = context
        self.handler = handler

    def prepare(self, name, text):
        self.sources = getWrapper(self.context, IExternalSourceManager)

    def __call__(self, tree):
        exported = self.handler.getExported()
        for node in tree.xpath(
                '//html:div[contains(@class, "external-source")]',
                namespaces={'html': 'http://www.w3.org/1999/xhtml'}):
            if 'data-source-instance' not in node.attrib:
                exported.reportProblem(
                    u'Broken source in document while exporting: '
                    u'Source parameters are missing.',
                    self.context)
                continue
            identifier = node.attrib['data-source-instance']
            del node.attrib['data-source-instance']

            try:
                source = self.sources(exported.request, instance=identifier)
            except SourceError as error:
                exported.reportProblem(
                    u'Broken source in document while exporting:'
                    u'{0}'.format(error),
                    self.context)
                continue
            if source.source is None:
                exported.reportProblem(
                    u'Broken source in document while exporting: '
                    u'source is no longer installed in the Silva site.',
                    self.context)
                continue
            node.attrib['source-identifier'] = source.getSourceId()

            # Fix this.
            producer = FieldProducer(self.context, self.handler, root=node)
            producer.startPrefixMapping(None, NS_SOURCE_URI)
            producer.startElement('fields')
            for serializer in getWrapper(
                source, IXMLFormSerialization).getSerializers():
                producer.startElement(
                    'field', {(None, 'id'): serializer.identifier})
                serializer(producer)
                producer.endElement('field')
            producer.endElement('fields')
            producer.endPrefixMapping(None)


class SourceParametersProducer(object):
    """ A Mixin class for exporting a source parameters.
    """

    def getHandler(self):
        return self

    def sax_source_parameters(self, source_manager):
        """`source_manager` should be a IExternalSourceManager bounded to
        an instance.
        """
        self.startElementNS(NS_SOURCE_URI, 'fields')
        for serializer in getWrapper(
                source_manager, IXMLFormSerialization).getSerializers():
            self.startElementNS(
                NS_SOURCE_URI,
                'field',
                {(None, 'id'): serializer.identifier})
            serializer(self)
            self.endElementNS(NS_SOURCE_URI, 'field')
        self.endElementNS(NS_SOURCE_URI, 'fields')


class SourceAssetProducer(producers.SilvaVersionedContentProducer):
    grok.adapts(ISourceAsset, Interface)

    def sax(self):
        self.startElementNS(NS_SOURCE_URI, 'source_asset',
            {'id': self.context.id})
        self.sax_workflow()
        self.sax_versions()
        self.endElementNS(NS_SOURCE_URI, 'source_asset')


class SourceAssetVersionProducer(producers.SilvaProducer,
                                 SourceParametersProducer):
    grok.adapts(ISourceAssetVersion, Interface)

    def sax(self):
        manager = self.context.get_controller(self.getExported().request)
        self.startElementNS(
            NS_SILVA_URI,
            'content',
            {'version_id': self.context.id,
             'source_identifier': manager.getSourceId()})
        self.sax_metadata()
        self.sax_source_parameters(manager)
        self.endElementNS(NS_SILVA_URI, 'content')





import unittest
import tempfile
import shutil
import os

from ..testing import FunctionalLayer
from ..CodeSourceService import CodeSourceInstallable
from ..CodeSourceService import CodeSourceExportable, InstallationError

TEST_DTML = """body { color: red };
"""

TEST_SOURCE = """[source]
id = source
title = Test Source
render_id = script
usuable = yes
previewable = yes
cacheable = no

"""

TEST_SCRIPT = """## Script (Python) "%s"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=model,version,REQUEST
##title=
##
return "Render source"
"""


class CodeSourceExportTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')
        self.directory = tempfile.mkdtemp('tests')
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addCodeSource('source', 'Test Source', 'script')
        self.root.source._fs_location = 'test:'

    def tearDown(self):
        shutil.rmtree(self.directory)

    def assertIsFile(self, *names):
        self.assertTrue(os.path.isfile(self.get_path(*names)))

    def assertIsDirectory(self, *names):
        self.assertTrue(os.path.isdir(self.get_path(*names)))

    def get_path(self, *names):
        return os.path.join(self.directory, *names)

    def test_python_script(self):
        """Test export a code source with a script.
        """
        # Add the rendering script
        factory = self.root.source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = self.root.source._getOb('script')
        script.write(TEST_SCRIPT % 'script')

        installable = CodeSourceInstallable('test:', self.directory)
        installable.export(self.root.source)

        self.assertItemsEqual(
            os.listdir(self.directory),
            ['parameters.xml', 'script.py', 'source.ini'])
        self.assertIsFile('script.py')
        self.assertIsFile('source.ini')
        self.assertIsFile('parameters.xml')
        with open(self.get_path('script.py'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SCRIPT % 'script')
        with open(self.get_path('source.ini'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SOURCE)

    def test_python_script_with_extension(self):
        """Test export a code source with a script with an extension.
        """
        # Add the rendering script
        factory = self.root.source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script.xml')
        script = self.root.source._getOb('script.xml')
        script.write(TEST_SCRIPT % 'script.xml')

        installable = CodeSourceInstallable('test:', self.directory)
        installable.export(self.root.source)

        self.assertItemsEqual(
            os.listdir(self.directory),
            ['parameters.xml', 'script.xml.py', 'source.ini'])
        self.assertIsFile('script.xml.py')
        self.assertIsFile('source.ini')
        self.assertIsFile('parameters.xml')
        with open(self.get_path('script.xml.py'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SCRIPT % 'script.xml')
        with open(self.get_path('source.ini'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SOURCE)

    def test_dtml_document(self):
        """Test export a code source with a DTML document with an invalid
        installable.

        """
        # Add the rendering script
        factory = self.root.source.manage_addProduct['OFS']
        factory.manage_addDTMLDocument('cool.css', 'Cool CSS')
        css = self.root.source._getOb('cool.css')
        css.munge(TEST_DTML)

        # Locations doesn't match, so the export fails.
        installable = CodeSourceInstallable('other:', '/')
        with self.assertRaises(InstallationError):
            installable.export(self.root.source)

        # Nothing got exported.
        self.assertItemsEqual(os.listdir(self.directory), [])

        # With an explicit location it will work, and not touch the installable.
        installable.export(self.root.source, directory=self.directory)

        self.assertItemsEqual(
            os.listdir(self.directory),
            ['parameters.xml', 'cool.css.dtml', 'source.ini'])
        self.assertIsFile('cool.css.dtml')
        self.assertIsFile('source.ini')
        self.assertIsFile('parameters.xml')
        with open(self.get_path('cool.css.dtml'), 'rb') as script:
            self.assertEqual(script.read(), TEST_DTML)
        with open(self.get_path('source.ini'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SOURCE)

    def test_folder(self):
        """Test export a code source via a not valid installable with a
        folder.
        """
        factory = self.root.source.manage_addProduct['OFS']
        factory.manage_addFolder('helpers')
        factory = self.root.source.helpers.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = self.root.source.helpers._getOb('script')
        script.write(TEST_SCRIPT % "script")
        # Add the rendering script
        factory = self.root.source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = self.root.source._getOb('script')
        script.write(TEST_SCRIPT % "script")

        installable = CodeSourceInstallable('test:', self.directory)
        self.assertFalse(installable.validate())
        installable.export(self.root.source)

        self.assertItemsEqual(
            os.listdir(self.directory),
            ['parameters.xml', 'script.py', 'helpers', 'source.ini'])
        self.assertIsDirectory('helpers')
        self.assertIsFile('helpers', 'script.py')
        self.assertIsFile('script.py')
        self.assertIsFile('source.ini')
        self.assertIsFile('parameters.xml')
        with open(self.get_path('source.ini'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SOURCE)
        with open(self.get_path('script.py'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SCRIPT % "script")
        with open(self.get_path('helpers', 'script.py'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SCRIPT % "script")

    def test_folder_export_only(self):
        """Test export only a code source containing a folder without using a
        full installable.
        """
        factory = self.root.source.manage_addProduct['OFS']
        factory.manage_addFolder('helpers')
        factory = self.root.source.helpers.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = self.root.source.helpers._getOb('script')
        script.write(TEST_SCRIPT % "script")
        # Add the rendering script
        factory = self.root.source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = self.root.source._getOb('script')
        script.write(TEST_SCRIPT % "script")

        CodeSourceExportable().export(self.root.source, self.directory)

        self.assertItemsEqual(
            os.listdir(self.directory),
            ['parameters.xml', 'script.py', 'helpers', 'source.ini'])
        self.assertIsDirectory('helpers')
        self.assertIsFile('helpers', 'script.py')
        self.assertIsFile('script.py')
        self.assertIsFile('source.ini')
        self.assertIsFile('parameters.xml')
        with open(self.get_path('source.ini'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SOURCE)
        with open(self.get_path('script.py'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SCRIPT % "script")
        with open(self.get_path('helpers', 'script.py'), 'rb') as script:
            self.assertEqual(script.read(), TEST_SCRIPT % "script")



def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CodeSourceExportTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import lxml.html
import logging

from five import grok
from zeam.component import getComponent
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.browser import TestRequest

from silva.core.views import views as silvaviews
from silva.core.editor.transform.base import TransformationFilter
from silva.core.editor.transform.editor.output import clean_editor_attributes
from silva.core.editor.transform.interfaces import IDisplayFilter
from silva.core.editor.transform.interfaces import IInputEditorFilter
from silva.core.editor.transform.interfaces import ISaveEditorFilter
from zeam.form import silva as silvaforms

from ..interfaces import IExternalSourceManager
from ..interfaces import ISourceEditableVersion
from ..errors import SourceError, ParametersMissingError
from ..editor.utils import parse_qs

logger = logging.getLogger('silva.externalsources')
SOURCE_XPATH = '//div[contains(@class, "external-source")]'


class ExternalSourceSaveFilter(TransformationFilter):
    """Process External Source information on save.
    """
    grok.implements(ISaveEditorFilter)
    grok.provides(ISaveEditorFilter)
    grok.order(20)
    grok.adapts(ISourceEditableVersion, IBrowserRequest)

    def prepare(self, name, text):
        self.manager = getComponent(
            self.context, IExternalSourceManager)(self.context)
        self.seen = set()

    def __call__(self, tree):
        for node in tree.xpath(SOURCE_XPATH):
            name = node.attrib.get('data-silva-name')
            instance = node.attrib.get('data-silva-instance')
            changed = 'data-silva-settings' in node.attrib
            parameters = parse_qs(node.attrib.get('data-silva-settings', ''))
            try:
                source = self.manager(
                    TestRequest(form=parameters), instance=instance, name=name)
            except SourceError:
                logger.error(
                    u'Broken source %s(%s) on content %s',
                    name, instance, '/'.join(self.context.getPhysicalPath()))
            else:
                if instance is None:
                    status = source.create()
                    instance = source.getId()
                elif changed:
                    status = source.save()
                if changed and status is silvaforms.FAILURE:
                    errors = {}
                    for error in source.errors:
                        if error.identifier != 'form':
                            errors[error.identifier] = error.title
                    logger.error(
                        u"Errors %s while saving source parameters %s "
                        u"for %s(%s) on content %s",
                        errors, parameters, name, instance,
                        '/'.join(self.context.getPhysicalPath()))
                node.attrib['data-source-instance'] = instance
                self.seen.add(instance)
            clean_editor_attributes(node)

    def finalize(self):
        # Remove all sources that we didn't see.
        for identifier in set(self.manager.all()).difference(self.seen):
            try:
                source = self.manager(self.request, instance=identifier)
                source.remove()
            except SourceError:
                logger.error(
                    'Error while removing source %s from text %s',
                    identifier, '/'.join(self.context.getPhysicalPath()))

    def truncate(self, name, text):
        manager = getComponent(
            self.context, IExternalSourceManager)(self.context)
        for identifier in manager.all():
            try:
                source = manager(self.request, instance=identifier)
                source.remove()
            except SourceError:
                logger.error(
                    'Error while removing source %s from text %s',
                    identifier, '/'.join(self.context.getPhysicalPath()))


class ExternalSourceInputFilter(TransformationFilter):
    """Updater External Source information on edit.
    """
    grok.implements(IInputEditorFilter)
    grok.provides(IInputEditorFilter)
    grok.order(20)
    grok.adapts(ISourceEditableVersion, IBrowserRequest)

    def __call__(self, tree):
        for node in tree.xpath(SOURCE_XPATH):
            instance = node.attrib.get('data-source-instance')
            if instance is not None:
                del node.attrib['data-source-instance']
            node.attrib['data-silva-instance'] = instance or ''


DEFAULT_CLASSES = set(['external-source', 'default'])

class ExternalSourceDisplayFilter(TransformationFilter):
    """Updater External Source information on edit.
    """
    grok.implements(IDisplayFilter)
    grok.provides(IDisplayFilter)
    grok.order(20)
    grok.adapts(ISourceEditableVersion, IBrowserRequest)

    def prepare(self, name, text):
        self.sources = getComponent(
            self.context, IExternalSourceManager)(self.context)

    def __call__(self, tree):
        for node in tree.xpath(SOURCE_XPATH):
            instance = node.attrib.get('data-source-instance')
            if instance is not None:
                del node.attrib['data-source-instance']
            try:
                if instance is None:
                    raise ParametersMissingError()
                source = self.sources(self.request, instance=instance)
                html = source.render()
                keep = not set(node.attrib['class'].split()).issubset(
                    DEFAULT_CLASSES)
            except SourceError, error:
                html = silvaviews.render(error, self.request).strip()
                if not html:
                    continue
                # There is already a class, as it is used to match
                # in the XPath.
                node.attrib['class'] += ' broken-source'
                keep = True

            result = lxml.html.fragment_fromstring(html, create_parent="div")
            if keep:
                node.extend(result)
            else:
                parent = node.getparent()
                if parent is not None:
                    index = parent.index(node)
                    for child in result:
                        parent.insert(index, child)
                        index += 1
                    parent.remove(node)




##parameters=model, REQUEST, version, ref, controller, autoplay, width, height, params=''

video_url = ref.absolute_url()
if width == '':
    width_attr = ''
else:
    width_attr = 'width="%s" ' % width

if height == '':
    height_attr = ''
else:
    height_attr = 'height="%s" ' % height

if controller == 'True':
    embed_controller = '1'
else:
    embed_controller = '0'

if autoplay== 'True':
    embed_autoplay = '1'
else:
    embed_autoplay = '0'


param_tags = ''
param_attrs = ''
if params:
    start = 0
    param_list = []
    while start < len(params):
        i = params.find('=', start)
        if i > -1:
            name = params[start:i].strip()
        else:
            break
        i = params.find('"', i) + 1
        j = params.find('"', i)
        if i > -1 and j > -1:
            value = params[i:j].strip()
        else:
            break
        param_list.append((name, value))
        start = j + 1
    for name, value in param_list:
        param_tags = param_tags + '\n  <param name="' + name \
                + '" value="' + value + '" />'
        param_attrs = param_attrs + '\n    ' + name + '="' + value + '" '

return """
<p class="p">
<object %s%s
  classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B"
  codebase="http://www.apple.com/qtactivex/qtplugin.cab">
  <param name="src" value="%s" />
  <param name="autoplay" value="%s" />
  <param name="controller" value="%s" />%s
  <comment>
  <embed %s%s autoplay="%s" controller="%s"
    src="%s"
    pluginspage="http://www.apple.com/quicktime/download/"%s />
  </comment>
</object>
</p>
""" % (width_attr, height_attr, video_url, embed_autoplay, embed_controller, param_tags,
       width_attr, height_attr, autoplay, controller, video_url, param_attrs)



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from zope.interface.verify import verifyObject
from zeam.component import getWrapper

from Products.Silva.testing import TestCase, TestRequest

from .. import errors
from ..interfaces import IExternalSourceController
from ..interfaces import IExternalSourceManager, IExternalSourceInstance
from ..testing import FunctionalLayer

HTML_WORKING_SOURCE = u"""
<div>
    <div class="external-source default"
         data-silva-name="cs_citation"
         data-silva-settings="field_citation=je bent een klootzak&amp;field_author=jou&amp;field_source=wikipedia">
    </div>
</div>
"""
HTML_BROKEN_SOURCE = u"""
<div>
    <div class="external-source default"
         data-silva-name="cs_data"
         data-silva-settings="field_data=je bent een klootzak&amp;source_failover=1">
    </div>
</div>
"""


class CreationTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')
        factory = self.root.manage_addProduct['silva.app.document']
        factory.manage_addDocument('example', 'Example')

    def test_create_transform(self):
        """Create a new source instance, of an existing code source.
        """
        request = TestRequest()
        version = self.root.example.get_editable()
        version.body.save(version, request, HTML_WORKING_SOURCE)

        # This gives access to all the sources
        sources = getWrapper(version, IExternalSourceManager)
        self.assertTrue(verifyObject(IExternalSourceManager, sources))
        self.assertEqual(len(sources.all()), 1)
        instance_key = list(sources.all())[0]

        parameters, source = sources.get_parameters(instance=instance_key)
        self.assertTrue(verifyObject(IExternalSourceInstance, parameters))

        # A parameters store data
        self.assertEqual(parameters.get_source_identifier(), 'cs_citation')
        self.assertEqual(parameters.get_parameter_identifier(), instance_key)
        self.assertEqual(source.id, 'cs_citation')
        self.assertEqual(parameters.citation, u'je bent een klootzak')
        self.assertEqual(parameters.author, u'jou')
        self.assertEqual(parameters.source, u'wikipedia')

        # You can bind parameters to a content and a request
        controller = sources(request, instance=instance_key)
        self.assertTrue(verifyObject(IExternalSourceController, controller))

    def test_create_transform_broken_failover(self):
        """Create a source that doesn't exists with failover. No
        source is created, but no error is yielded.
        """
        request = TestRequest()
        version = self.root.example.get_editable()
        version.body.save(version, request, HTML_BROKEN_SOURCE)

        # This gives access to all the sources
        sources = getWrapper(version, IExternalSourceManager)
        self.assertTrue(verifyObject(IExternalSourceManager, sources))
        self.assertEqual(len(sources.all()), 0)

    def test_create(self):
        """Create a source by hand.
        """
        request = TestRequest(form={
                'field_citation': 'je bent een hero',
                'field_author': 'u',
                'field_source': 'google',
                })
        version = self.root.example.get_editable()
        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 0)

        controller = sources(request, name='cs_citation')
        # You cannot save or remove the source, it does not exist yet.
        with self.assertRaises(AssertionError):
            controller.save()
        with self.assertRaises(AssertionError):
            controller.remove()
        self.assertEqual(controller.getId(), None)

        # But you can create it, and it will created after.
        controller.create()
        self.assertNotEqual(controller.getId(), None)
        self.assertEqual(len(sources.all()), 1)
        instance_key = list(sources.all())[0]

        parameters, source = sources.get_parameters(instance=instance_key)
        self.assertTrue(verifyObject(IExternalSourceInstance, parameters))
        self.assertEqual(controller.getId(), instance_key)
        self.assertEqual(parameters.get_parameter_identifier(), instance_key)
        self.assertEqual(parameters.get_source_identifier(), 'cs_citation')
        self.assertEqual(source.id, 'cs_citation')
        self.assertEqual(parameters.citation, u'je bent een hero')
        self.assertEqual(parameters.author, u'u')
        self.assertEqual(parameters.source, u'google')

    def test_create_broken(self):
        """Create a broken source: you can't access the create method.
        """
        request = TestRequest()
        version = self.root.example.get_editable()

        # This gives access to all the sources
        sources = getWrapper(version, IExternalSourceManager)
        self.assertTrue(verifyObject(IExternalSourceManager, sources))
        self.assertEqual(len(sources.all()), 0)

        with self.assertRaises(errors.SourceMissingError):
            sources.get_parameters()
        with self.assertRaises(errors.SourceMissingError):
            sources.get_parameters(name='cs_nonexisting')
        with self.assertRaises(errors.SourceMissingError):
            sources(request, name='cs_nonexisting')

        with self.assertRaises(errors.ParametersMissingError):
            sources.get_parameters(instance='nonexisting')
        with self.assertRaises(errors.ParametersMissingError):
            sources(request, instance='nonexisting')


class WorkingControllerTestCase(TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')
        factory = self.root.manage_addProduct['silva.app.document']
        factory.manage_addDocument('example', 'Example')

        version = self.root.example.get_editable()
        version.body.save(version, TestRequest(), HTML_WORKING_SOURCE)
        self.sources = getWrapper(version, IExternalSourceManager)
        self.identifier = list(self.sources.all())[0]

    def test_controller(self):
        """Test the controller API.
        """
        controller = self.sources(TestRequest(), instance=self.identifier)
        _, source = self.sources.get_parameters(instance=self.identifier)
        self.assertTrue(verifyObject(IExternalSourceController, controller))

        self.assertEqual(controller.getId(), self.identifier)
        self.assertEqual(controller.getSourceId(), 'cs_citation')
        self.assertEqual(controller.editable(), True) # The source have fields.
        self.assertEqual(controller.label, source.get_title())
        self.assertEqual(controller.description, source.get_description())

    def test_create(self):
        """You cannot recreate an existing source.
        """
        controller = self.sources(TestRequest(), instance=self.identifier)
        with self.assertRaises(AssertionError):
            controller.new()

    def test_render(self):
        """Render a defined source.
        """
        controller = self.sources(TestRequest(), instance=self.identifier)
        self.assertXMLEqual(controller.render(), """
<div class="citation">
 je bent een klootzak
 <div class="author">jou</div>
 <div class="source">wikipedia</div>
</div>
""")

        # If you set a source template, it is used around the output
        parameters, _ = self.sources.get_parameters(instance=self.identifier)
        parameters.set_source_template("""<div class="portlet">
   <!-- source output -->
   <div class="portlet-footer" />
</div>
""")
        self.assertXMLEqual(controller.render(), """
<div class="portlet">
 <div class="citation">
   je bent een klootzak
   <div class="author">jou</div>
   <div class="source">wikipedia</div>
 </div>
 <div class="portlet-footer" />
</div>
""")


    def test_remove(self):
        """Remove a defined source.
        """
        controller = self.sources(TestRequest(), instance=self.identifier)
        controller.remove()

        self.assertEqual(len(self.sources.all()), 0)
        with self.assertRaises(errors.ParametersMissingError):
            self.sources.get_parameters(instance=self.identifier)
        with self.assertRaises(errors.ParametersMissingError):
            self.sources(TestRequest(), instance=self.identifier)

    def test_save(self):
        """Updating source parameters.
        """
        request = TestRequest(form={
                'field_citation': 'il fait soleil',
                'field_author': 'moi',
                'marker_field_citation': '1',
                'marker_field_author': '1',
                'marker_field_source': '1'})
        controller = self.sources(request, instance=self.identifier)
        controller.save()

        parameters, source = self.sources.get_parameters(
            instance=self.identifier)

        self.assertEqual(parameters.get_source_identifier(), 'cs_citation')
        self.assertEqual(parameters.citation, u'il fait soleil')
        self.assertEqual(parameters.author, u'moi')
        self.assertEqual(parameters.source, u'')
        self.assertXMLEqual(controller.render(), """
<div class="citation">
 il fait soleil
 <div class="author">moi</div>
 <div class="source"></div>
</div>
""")

    def test_save_unicode(self):
        """Updating source parameters, with unicode values.
        """
        request = TestRequest(form={
                'field_citation': u'Cela est éternel'.encode('utf-8'),
                'field_author': 'moi',
                'marker_field_citation': '1',
                'marker_field_author': '1',
                'marker_field_source': '1'})
        controller = self.sources(request, instance=self.identifier)
        controller.save()

        parameters, source = self.sources.get_parameters(
            instance=self.identifier)

        self.assertEqual(parameters.get_source_identifier(), 'cs_citation')
        self.assertEqual(parameters.citation, u'Cela est éternel')
        self.assertEqual(parameters.author, u'moi')
        self.assertEqual(parameters.source, u'')
        self.assertXMLEqual(controller.render(), u"""
<div class="citation">
 Cela est éternel
 <div class="author">
  moi
 </div>
 <div class="source">
 </div>
</div>
""")


class BrokenControllerTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')
        factory = self.root.manage_addProduct['silva.app.document']
        factory.manage_addDocument('example', 'Example')

        # Create cs_data as a copy of cs_citation
        token = self.root.manage_copyObjects(['cs_citation'])
        self.root.manage_pasteObjects(token)
        self.root.manage_renameObject('copy_of_cs_citation', 'cs_data')

        # Create source.
        version = self.root.example.get_editable()
        version.body.save(version, TestRequest(), HTML_BROKEN_SOURCE)
        self.sources = getWrapper(version, IExternalSourceManager)
        self.identifier = list(self.sources.all())[0]

        # Remove source, that will break the instance.
        self.root.manage_delObjects(['cs_data'])

    def test_render(self):
        """Render a broken source, you will get an error.
        """
        controller = self.sources(TestRequest(), instance=self.identifier)
        with self.assertRaises(errors.SourceMissingError):
            controller.render()

    def test_remove(self):
        """Remove a broken source, this should work.
        """
        controller = self.sources(TestRequest(), instance=self.identifier)
        controller.remove()

        self.assertEqual(len(self.sources.all()), 0)
        with self.assertRaises(errors.ParametersMissingError):
            self.sources.get_parameters(instance=self.identifier)
        with self.assertRaises(errors.ParametersMissingError):
            self.sources(TestRequest(), instance=self.identifier)

    def test_save(self):
        """You cannot save values into a broken source.
        """
        request = TestRequest(
            form={'field_citation': 'il fait soleil',
                  'field_author': 'moi'})
        controller = self.sources(request, instance=self.identifier)
        with self.assertRaises(errors.SourceMissingError):
            controller.save()


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CreationTestCase))
    suite.addTest(unittest.makeSuite(WorkingControllerTestCase))
    suite.addTest(unittest.makeSuite(BrokenControllerTestCase))
    return suite




# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from cStringIO import StringIO
import csv
import os

from five import grok
from zope import schema
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

# Zope
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from OFS.Folder import Folder

from Products.Formulator.Form import ZMIForm
from Products.Formulator.XMLToForm import XMLToForm
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

# Silva
from Products.Silva import SilvaPermissions
from Products.Silva.Asset import Asset
from Products.Silva.Asset import AssetEditTab
from Products.SilvaExternalSources.ExternalSource import EditableExternalSource
from Products.SilvaExternalSources.interfaces import ICSVSource
from Products.SilvaMetadata.interfaces import IMetadataService

from silva.core.interfaces import IVersion
from silva.core.views import views as silvaviews
from silva.core.conf import schema as silvaschema
from silva.core.conf.interfaces import ITitledContent
from silva.core import conf as silvaconf
from silva.translations import translate as _
from zeam.form import silva as silvaforms
from zeam.utils.batch import Batch, IBatching

DATA_PATH = os.path.join(os.path.dirname(__file__), 'layout')


# Asset must be used inherited before EditableExternalSource
class CSVSource(Folder, Asset, EditableExternalSource):
    """CSV Source is an asset that displays tabular data from a
    spreadsheet or database. The format of the uploaded text file
    should be &#8216;comma separated values&#8217;. The asset can
    be linked directly, or inserted in a document with the External
    Source element. If necessary, all aspects of the display can be
    customized in the rendering templates of the CSV Source.
    """
    grok.implements(ICSVSource)

    meta_type = "Silva CSV Source"
    security = ClassSecurityInfo()

    _layout_id = 'layout'
    _default_batch_size = 20

    # register priority, icon and factory
    silvaconf.priority(1)
    silvaconf.icon('www/csvsource.png')

    def __init__(self, id):
        super(CSVSource, self).__init__(id)
        self._raw_data = ''
        self._data = []

    # ACCESSORS

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'to_html')
    def to_html(self, content, request, **parameters):
        """ render HTML for CSV source
        """
        rows = self._data[:]
        param = {}
        param.update(parameters)
        if not param.get('csvtableclass'):
            param['csvtableclass'] = 'default'
        batch_size = self._default_batch_size
        batch = ''
        if param.get('csvbatchsize'):
            batch_size = int(param.get('csvbatchsize'))
        model = content
        if IVersion.providedBy(content):
            model = content.get_content()
        if rows:
            headings = rows[0]
            rows = Batch(
                rows[1:],
                count=batch_size, name=self.getId(), request=request)
            param['headings'] = headings
            batch = getMultiAdapter((model, rows, request), IBatching)()
        return self.layout(table=rows, batch=batch, parameters=param)

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_file')
    def get_file(self):
        """Return the file content.
        """
        return self._raw_data

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_file_size')
    def get_file_size(self):
        """Get the size of the file as it will be downloaded.
        """
        if self._raw_data:
            return len(self._raw_data)
        return 0

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_mime_type')
    def get_mime_type(self):
        return 'text/csv'

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_filename')
    def get_filename(self):
        return self.getId() + '.csv'

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'get_table_class')
    def get_table_class(self):
        """Returns css class for table """
        return self._table_class

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_description')
    def get_description(self):
        """ Return desc from meta-data system"""
        ms = self.service_metadata
        return ms.getMetadataValue(self, 'silva-extra', 'content_description')

    # MODIFIERS

    def _update_data(self, data):

        def convert_to_unicode(line):
            return map(
                lambda v: v.decode(self._data_encoding, 'replace'),
                line)

        try:
            csv_data = map(convert_to_unicode, csv.reader(StringIO(data)))
        except csv.Error as error:
            raise ValueError(u"Invalid CSV file: %s" % error.args[0])

        self._data = csv_data
        self._raw_data = data
        notify(ObjectModifiedEvent(self))

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'set_file')
    def set_file(self, file):
        return self._update_data(file.read())

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'set_data_encoding')
    def set_data_encoding(self, encoding):
        self._data_encoding = encoding
        self._update_data(self._raw_data)

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'set_table_class')
    def set_table_class(self, css_class):
        self._table_class = css_class

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'set_description')
    def set_description(self, desc):
        if not isinstance(desc, unicode):
            desc = desc.encode('utf-8')

        binding = getUtility(IMetadataService).getMetadata(self)
        binding.setValues('silva-extra', {'content_description' : desc})


InitializeClass(CSVSource)


def reset_parameter_form(source):
    with open(os.path.join(DATA_PATH, 'csvparameters.xml')) as data:
        form = ZMIForm('form', 'Parameters form', unicode_mode=1)
        XMLToForm(data.read(), form)
        source.set_form(form)

def reset_table_layout(source):
    # Works for Zope object implementing a 'write()" method...
    layout = [
        ('layout', ZopePageTemplate, 'csvtable.zpt'),
    ]

    for identifier, factory, filename in layout:
        with open(os.path.join(DATA_PATH, filename), 'rb') as data:
            if identifier not in source.objectIds():
                source._setObject(identifier, factory(identifier))
            source._getOb(identifier).write(data.read())

@grok.subscribe(ICSVSource, IObjectCreatedEvent)
def source_created(source, event):
    reset_table_layout(source)
    reset_parameter_form(source)

@grok.subscribe(ICSVSource, IObjectModifiedEvent)
def source_modified(source, event):
    source.update_quota()


@apply
def encoding_source():
    encodings = []
    for title, key in [
        ('ISO-8859-15 (Western Europe - Latin-1/EURO)', 'ISO-8859-15'),
        ('ISO-8859-1 (Western Europe - Latin-1)', 'ISO-8859-1'),
        ('Windows-1252 (Western Europe - Latin-1/Windows)', 'cp1252'),
        ('Mac Roman (Western Europe - Apple Macintosh)', 'mac_roman'),
        ('UTF-8 (Unicode)', 'utf-8')]:
        encodings.append(SimpleTerm(value=key, token=key, title=title))
    return SimpleVocabulary(encodings)


class ICSVSourceFields(ITitledContent):

    file = silvaschema.Bytes(
        title=_(u"file"),
        description=_(u"File to upload as source data."),
        required=True)
    data_encoding = schema.Choice(
        title=_(u"character encoding"),
        description=_(u'Character encoding of the source data.'),
        source=encoding_source,
        required=False)


class CSVSourceAddForm(silvaforms.SMIAddForm):
    """CSVSource Add Form.
    """
    grok.context(ICSVSource)
    grok.name(u'Silva CSV Source')

    fields = silvaforms.Fields(ICSVSourceFields)


class CSVSourceEditForm(silvaforms.SMISubForm):
    """CSVSource Edit Form.
    """
    grok.view(AssetEditTab)
    grok.order(10)

    label = _(u'Edit file content')
    ignoreContent = False
    dataManager = silvaforms.SilvaDataManager

    fields = silvaforms.Fields(ICSVSourceFields).omit('id')
    fields['file'].fileSetLabel = _(
        u"Click the Upload button to replace the current CSV with a new one.")
    actions  = silvaforms.Actions(
        silvaforms.CancelEditAction(),
        silvaforms.EditAction())


class CSVSourceView(silvaviews.View):
    """View a CSVSource.
    """
    grok.context(ICSVSource)

    def render(self):
        return self.content.to_html(self.context, self.request)




import unittest
import shutil
import tempfile
import os

from zope.interface.verify import verifyObject

from ..interfaces import ICodeSource
from ..testing import FunctionalLayer
from ..CodeSourceService import CodeSourceInstallable, InstallationError


class CodeSourceImportTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')
        self.directory = tempfile.mkdtemp('tests')
        shutil.copytree(
            self.layer.get_fixture('cs_fancytest'),
            os.path.join(self.directory, 'cs_fancytest'))
        with open(self.get_path('.DS_Store'), 'wb') as test:
            test.write('test')
        with open(self.get_path('._source.ini'), 'wb') as test:
            test.write('test')

    def tearDown(self):
        shutil.rmtree(self.directory)

    def get_path(self, *names):
        return os.path.join(self.directory, 'cs_fancytest', *names)

    def test_import(self):
        installable = CodeSourceInstallable('test:', self.get_path())
        installable.install(self.root)

        source = self.root._getOb('cs_fancytest', None)
        self.assertTrue(verifyObject(ICodeSource, source))
        self.assertItemsEqual(
            source.objectIds(),
            ['css', 'feedback', 'js', 'README', 'script'])
        self.assertItemsEqual(
            source.css.objectIds(),
            ['advanced.css'])
        self.assertItemsEqual(
            source.js.objectIds(),
            ['advanced.js'])

    def test_import_already_exist(self):
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addFolder('cs_fancytest', 'Fancy duplicate')

        installable = CodeSourceInstallable('test:', self.get_path())
        # The identifier is already in use, so you get an error.
        with self.assertRaises(InstallationError):
            installable.install(self.root)

    def test_import_export_and_update(self):
        installable = CodeSourceInstallable(
            'test:', self.get_path())
        # Return True upon success
        self.assertTrue(installable.install(self.root))

        source = self.root._getOb('cs_fancytest', None)
        self.assertTrue(verifyObject(ICodeSource, source))
        self.assertItemsEqual(
            source.objectIds(),
            ['css', 'feedback', 'js', 'README', 'script'])
        self.assertItemsEqual(
            source.css.objectIds(),
            ['advanced.css'])
        self.assertItemsEqual(
            source.js.objectIds(),
            ['advanced.js'])

        # Call again install and it will return False, before it is
        # already installed.
        self.assertFalse(installable.install(self.root))

        # We export again the source.
        installable.export(source)
        self.assertItemsEqual(
            os.listdir(self.get_path()),
            ['css', 'feedback.xml', 'js', 'parameters.xml',
             'README.txt', 'script.pt', 'source.ini'])
        self.assertItemsEqual(
            os.listdir(self.get_path('css')),
            ['advanced.css.dtml'])
        self.assertItemsEqual(
            os.listdir(self.get_path('js')),
            ['advanced.js'])

        # And update it
        self.assertTrue(installable.update(source, purge=True))
        self.assertItemsEqual(
            source.objectIds(),
            ['css', 'feedback', 'js', 'README', 'script'])
        self.assertItemsEqual(
            source.css.objectIds(),
            ['advanced.css'])
        self.assertItemsEqual(
            source.js.objectIds(),
            ['advanced.js'])

    def test_rename_file_and_update(self):
        installable = CodeSourceInstallable('test:', self.get_path())
        self.assertTrue(installable.install(self.root))
        source = self.root._getOb('cs_fancytest', None)
        self.assertItemsEqual(
            source.objectIds(),
            ['css', 'feedback', 'js', 'README', 'script'])

        os.rename(
            self.get_path('README.txt'),
            self.get_path('RENAMED_README.txt'))
        self.assertTrue(installable.update(source, purge=True))
        self.assertItemsEqual(
            source.objectIds(),
            ['css', 'feedback', 'js', 'RENAMED_README', 'script'])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CodeSourceImportTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import logging

from silva.core import conf as silvaconf
from silva.core.conf.installer import DefaultInstaller
from zope.interface import Interface

silvaconf.extension_name('SilvaExternalSources')
silvaconf.extension_title('Silva External Sources')
silvaconf.extension_depends(["Silva", "silva.core.editor"])
silvaconf.extension_default()

logger = logging.getLogger('silva.externalsources')


class IExtension(Interface):
    """Silva External Sources extension.
    """

class ExternalSourcesInstaller(DefaultInstaller):
    """Silva External Sources installer
    """

    def install_custom(self, root):
        installed_ids = root.objectIds()
        if 'service_codesources' not in installed_ids:
            factory = root.manage_addProduct['SilvaExternalSources']
            factory.manage_addCodeSourceService()

        service = root.service_codesources
        for identifier in ['cs_toc', 'cs_citation',]:
            if identifier not in installed_ids:
                candidates = list(
                    service.get_installable_source(identifier=identifier))
                if len(candidates) == 1:
                    candidates[0].install(root)
                else:
                    logger.error(
                        u"Could not find default source %s to install it.",
                        identifier)


    def uninstall_custom(self, root):
        installed_ids = root.objectIds()
        if 'service_codesources' in installed_ids:
            root.manage_delObjects(['service_codesources'])


install = ExternalSourcesInstaller('SilvaExternalSources', IExtension)


CLASS_CHANGES = {
    'Products.SilvaExternalSources.editor.instance SourceParameters':
        'Products.SilvaExternalSources.SourceInstance SourceParameters',
    'Products.SilvaExternalSources.editor.instance SourceInstances':
        'Products.SilvaExternalSources.SourceInstance SourceInstances'
    }




from AccessControl import ModuleSecurityInfo
from AccessControl.security import checkPermission
from silva.app.document.interfaces import IDocument
from silva.core.interfaces import IAddableContents, IPublishable, ITreeContents
from silva.core.interfaces import IAutoTOC
from silva.fanstatic import Group, Snippet, ExternalResource
from zope.component import getMultiAdapter, queryMultiAdapter
from zope.publisher.interfaces.browser import IBrowserRequest

module_security = ModuleSecurityInfo(
    'Products.SilvaExternalSources.codesources.api')


module_security.declarePublic('render_content')
def render_content(content, request, suppress_title=False):
    """Render a content for inclusion.
    """
    if not (checkPermission('zope2.View', content)
            or IBrowserRequest.providedBy(request)):
        # You can't see the content or don't have a valid request.
        return u''
    content = content.get_silva_object()
    if suppress_title:
        if IDocument.providedBy(content):
            version = content.get_viewable()
            if version is None:
                return u''
            details = getMultiAdapter((version, request), name="details")
            return details.get_text()
        if IAutoTOC.providedBy(content):
            toc = getMultiAdapter((content, request), name="toc")
            toc.update()
            return toc.render()
        # suppress title is not supported for other contents, render them publicly.
    renderer = queryMultiAdapter((content, request), name='content.html')
    if renderer is not None:
        return renderer()
    return u''


def _include_resources(factory, resources, category, requires, bottom):

    def create(resource, depends):
        return [factory(resource, category=category,
                        depends=depends, bottom=bottom)]

    # If the ordering in fanstatic would not randomize the relative
    # order of the resources the code would be:
    # if len(resources) == 1:
    #     result = create(resources[0], requires)
    # else:
    #     result = Group(map(lambda r: create(r, requires), resources))

    # But instead we have to make each resource depend on each other.
    for resource in resources:
        requires = create(resource, requires)
    result = requires[0]

    result.need()
    return result


module_security.declarePublic('include_resource')
def include_resource(css=None, js=None, requires=[], bottom=False):
    """Add a Javascript or CSS to the document head. It can depends on
    other resources, like for instance jquery::

      include_resource(js='http://url', requires=['jquery'])

    It returns a resource that can be used as dependencies again.
    """
    if css is None:
        if not isinstance(js, (list, tuple)):
            if js is None:
                raise ValueError("Resources to include are missing")
            resources = [js]
        else:
            resources = js
        category = 'js'
    else:
        if not isinstance(css, (list, tuple)):
            resources = [css]
        else:
            resources = css
        category = 'css'
    return _include_resources(
        ExternalResource, resources, category, requires, bottom)


module_security.declarePublic('include_snippet')
def include_snippet(css=None, js=None, requires=[], bottom=False):
    """Include a Javascript or CSS snippet in the document head. It
    can depends on other resources, like for instance jquery::

      include_resource(js='alert("I like JS !");', requires=['jquery'])

    It returns a resource that can be used as dependencies again.
    """
    if css is None:
        if js is None:
            raise ValueError("Snippet to include is missing")
        resources = [js]
        category = 'js'
    else:
        resources = [css]
        category = 'css'
    return _include_resources(
        Snippet, resources, category, requires, bottom)


module_security.declarePublic('get_publishable_content_types')
def get_publishable_content_types(context):
    """Return meta_types of content that can be published in the Silva
    site.
    """
    container = context.get_root()
    return IAddableContents(container).get_all_addables(require=IPublishable)


module_security.declarePublic('get_container_content_types')
def get_container_content_types(context):
    """Return meta_types of all content can be used in the Silva site.
    """
    container = context.get_root()
    return IAddableContents(container).get_all_addables()


module_security.declarePublic('get_content_tree')
def get_content_tree(context, depth):
    # This is extremely slow and performance-wise bad
    return ITreeContents(context).get_tree(depth)


module_security.declarePublic('get_content_public_tree')
def get_content_public_tree(context, depth):
    # This is extremely slow and performance-wise bad
    return ITreeContents(context).get_public_tree(depth)






# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt


from Products.Silva.testing import SilvaLayer
import Products.SilvaExternalSources
import transaction


class SilvaExternalSourcesLayer(SilvaLayer):
    default_products = SilvaLayer.default_products + [
        'SilvaExternalSources',
        'ZSQLiteDA'
        ]
    default_packages = SilvaLayer.default_packages + [
        'silva.app.document'
        ]

    def _install_application(self, app):
        super(SilvaExternalSourcesLayer, self)._install_application(app)
        app.root.service_extensions.install('silva.app.document')
        app.root.service_extensions.install('SilvaExternalSources')
        transaction.commit()


FunctionalLayer = SilvaExternalSourcesLayer(Products.SilvaExternalSources)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from Acquisition import aq_get
from AccessControl.SecurityManagement import getSecurityManager


def _exec(self, bound_names, args, kw):
    """Call a Page Template"""
    if 'args' not in kw:
        kw['args'] = args
    bound_names['options'] = kw
    request = kw.get('REQUEST')
    if request is None:
        request = aq_get(self, 'REQUEST', None)
        if request is not None:
            response = request.response
            if not response.headers.has_key('content-type'):
                response.setHeader('content-type', self.content_type)
    else:
        del kw['REQUEST']
        bound_names['request'] = request

    security = getSecurityManager()
    bound_names['user'] = security.getUser()

    # Retrieve the value from the cache.
    keyset = None
    if self.ZCacheable_isCachingEnabled():
        # Prepare a cache key.
        keyset = {'here': self._getContext(),
                  'bound_names': bound_names}
        result = self.ZCacheable_get(keywords=keyset)
        if result is not None:
            # Got a cached value.
            return result

    # Execute the template in a new security context.
    security.addContext(self)

    try:
        result = self.pt_render(extra_context=bound_names)
        if keyset is not None:
            # Store the result in the cache.
            self.ZCacheable_set(result, keywords=keyset)
        return result
    finally:
        security.removeContext(self)



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import collections
import os
import re
import logging
import ConfigParser
import shutil
import pkg_resources
import operator

from datetime import datetime
from pkg_resources import iter_entry_points

from Acquisition import aq_parent
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from OFS.interfaces import IObjectWillBeRemovedEvent
from ZODB.broken import Broken

from Products.Formulator.Form import ZMIForm
from Products.Formulator.FormToXML import formToXML
from Products.Silva.ExtensionRegistry import extensionRegistry

from five import grok
from zope.interface import Interface
from zope import schema
from zope.component import getUtility, queryUtility
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent.interfaces import IObjectAddedEvent
from js.jquery import jquery

from silva.core import conf as silvaconf
from silva.core.interfaces import IContainer, ISilvaConfigurableService
from silva.core.interfaces import IMimeTypeClassifier
from silva.core.messages.interfaces import IMessageService
from silva.core.services.base import SilvaService
from silva.core.services.utils import walk_silva_tree
from silva.core.views import views as silvaviews
from silva.fanstatic import need
from silva.translations import translate as _
from silva.ui import rest, menu
from zeam.form import silva as silvaforms

from .interfaces import ICodeSource, ICodeSourceService, ICodeSourceInstaller
from .interfaces import ISourceErrors

logger = logging.getLogger('silva.externalsources')


CONFIGURATION_FILE = 'source.ini'
PARAMETERS_FILE = 'parameters.xml'
# All the code following are default helper to help you to install
# your code-sources packaged on the file system.


class Exporter(object):

    def __init__(self, content):
        self.content = content

    def get_path(self, identifier):
        raise NotImplemented

    def __call__(self, path):
        raise NotImplemented


class PageTemplateExporter(Exporter):
    """Export a page template. If the name of the template is in upper
    case this will create a .txt file, a .pt otherwise.
    """

    def get_path(self, identifier):
        if '.' not in identifier:
            if identifier.isupper():
                return identifier + '.txt'
            else:
                return identifier + '.pt'
        return identifier

    def __call__(self, path):
        with open(path, 'wb') as target:
            data = self.content.read()
            if isinstance(data, unicode):
                data = data.encode('utf-8')
            if not data.endswith(os.linesep):
                data += os.linesep
            target.write(data)


class FileExporter(Exporter):
    """Export a file or an image. If identifier doesn't contains an
    extension, guess one from the filename.
    """

    def get_path(self, identifier):
        if '.' not in identifier:
            guess_extension = getUtility(IMimeTypeClassifier).guess_extension
            return identifier + guess_extension(self.content.content_type)
        return identifier

    def __call__(self, path):
        with open(path, 'wb') as target:
            data = self.content.data
            if isinstance(data, basestring):
                target.write(data)
            else:
                while data is not None:
                    target.write(data.data)
                    data = data.next


class ScriptExporter(PageTemplateExporter):

    def get_path(self, identifier):
        return identifier + '.py'


class DTMLExporter(Exporter):

    def get_path(self, identifier):
        return identifier + '.dtml'

    def __call__(self, path):
        with open(path, 'wb') as target:
            target.write(self.content.raw)


class FormulatorExporter(Exporter):

    def get_path(self, identifier):
        return identifier + '.xml'

    def __call__(self, path):
        with open(path, 'w') as target:
            target.write(formToXML(self.content))


class FolderExporter(Exporter):

    def get_path(self, identifier):
        return identifier

    def __call__(self, path):
        if not os.path.exists(path):
            os.makedirs(path)
            existing = []
        else:
            existing = os.listdir(path)
        exported = []
        for identifier, content in self.content.objectItems():
            factory = EXPORTERS.get(content.meta_type, None)
            if factory is None:
                logger.info(
                    u"don't know how to export %s for code source %s",
                    content.meta_type, self.identifier)
                continue
            exporter = factory(content)
            filename = exporter.get_path(identifier)
            try:
                exporter(os.path.join(path, filename))
            except:
                logger.error(
                    "failed to export %s for code source %s",
                    content.meta_type, self.identifier)
            exported.append(filename)
        for filename in existing:
            if filename not in exported:
                fullname = os.path.join(path, filename)
                if os.path.isdir(fullname):
                    shutil.rmtree(fullname)
                else:
                    os.unlink(fullname)


class ExternalMethodExporter(Exporter):

    def get_path(self, identifier):
        return identifier + ".em"

    def __call__(self, path):
        with open(path, 'wb') as target:
            target.write('title:string=%s\n' % self.content.title)
            target.write('module:string=%s\n' % self.content._module)
            target.write('function:string=%s\n' % self.content._function)


class Importer(object):
    keep_extension = False

    def __call__(self, context, identifier, path):
        raise NotImplemented


class PageTemplateImporter(Importer):
    """Install a page template.
    """

    def __call__(self, context, identifier, path):
        with open(path, 'rb') as data:
            factory = context.manage_addProduct['PageTemplates']
            factory.manage_addPageTemplate(identifier, '', data.read())


class FileImporter(Importer):
    """Install a File.
    """
    keep_extension = True

    def __call__(self, context, identifier, path):
        with open(path, 'rb') as data:
            factory = context.manage_addProduct['OFSP']
            factory.manage_addFile(identifier, file=data)


class ImageImporter(FileImporter):
    """Install an Image.
    """

    def __call__(self, context, identifier, path):
        with open(path, 'rb') as data:
            factory = context.manage_addProduct['OFSP']
            factory.manage_addImage(identifier, file=data)


class PythonScriptImporter(Importer):
    """Install a Python script.
    """

    def __call__(self, context, identifier, path):
        with open(path, 'rb') as data:
            factory = context.manage_addProduct['PythonScripts']
            factory.manage_addPythonScript(identifier)
            script = context._getOb(identifier)
            script.write(data.read())


class DTMLImporter(Importer):

    def __call__(self, context, identifier, path):
        with open(path, 'rb') as data:
            factory = context.manage_addProduct['OFS']
            factory.manage_addDTMLDocument(identifier)
            dtml = context._getOb(identifier)
            dtml.munge(data.read())


class FormulatorImporter(Importer):

    def __call__(self, context, identifier, path):
        with open(path, 'rb') as data:
            form = ZMIForm(identifier, 'Parameters form')
            try:
                form.set_xml(data.read())
            except:
                logger.exception('''
                    Error while installing Formulator form id "%s" in "%s"
                    ''' % (identifier, '/'.join(context.getPhysicalPath())))
            else:
                if identifier == 'parameters':
                    context.set_form(form)
                else:
                    context._setObject(identifier, form)


class FolderOrFileImporter(Importer):
    keep_extension = True

    def __call__(self, context, identifier, path):
        factory = context.manage_addProduct['OFSP']
        if os.path.isdir(path):
            factory.manage_addFolder(identifier)
            container = context._getOb(identifier)
            for filename in os.listdir(path):
                if filename.startswith('.'):
                    continue
                identifier, extension = os.path.splitext(filename)
                factory = INSTALLERS.get(extension, None)
                if factory is None:
                    # Default to None, file default installer.
                    factory = INSTALLERS[None]
                if factory.keep_extension:
                    identifier = filename
                installer = factory()
                if identifier in container.objectIds():
                    container.manage_delObjects([identifier])
                installer(container, identifier,
                          os.path.join(path, filename))
        else:
            with open(path, 'rb') as data:
                factory.manage_addFile(identifier, file=data)


EXPORTERS = {
    'File': FileExporter,
    'Image': FileExporter,
    'Script (Python)': ScriptExporter,
    'Page Template': PageTemplateExporter,
    'Folder': FolderExporter,
    'DTML Document': DTMLExporter,
    'DTML Method': DTMLExporter,
    'External Method': ExternalMethodExporter,
    'Formulator Form': FormulatorExporter,
    }
INSTALLERS = {
    '.png': ImageImporter,
    '.gif': ImageImporter,
    '.jpeg': ImageImporter,
    '.jpg': ImageImporter,
    '.pt': PageTemplateImporter,
    '.txt': PageTemplateImporter,
    '.py': PythonScriptImporter,
    '.xml': FormulatorImporter,
    '.dtml': DTMLImporter,
    None: FolderOrFileImporter, }  # None is the default installer.


class InstallationError(ValueError):
    pass


class CodeSourceExportable(object):

    def __init__(self):
        self._config = ConfigParser.ConfigParser()

    def _get_installables(self):
        return []

    def validate(self):
        """Return true if the definition is complete.
        """
        valid = True
        if not self._config.has_section('source'):
            valid = False
        elif not self._config.has_option('source', 'id'):
            valid = False
        elif not self._config.has_option('source', 'title'):
            valid = False
        elif not self._config.has_option('source', 'render_id'):
            valid = False
        return valid

    def export(self, source, directory):
        assert ICodeSource.providedBy(source)

        # Step 1, export configuration.
        if not self._config.has_section('source'):
            self._config.add_section('source')

        def set_value(key, value):
            if value:
                self._config.set('source', key, value)
            elif self._config.has_option('source', key):
                self._config.remove_option('source', key)

        set_value('id', source.getId())
        set_value('title', source.get_title())
        set_value('description', source.get_description())
        set_value('render_id', source.get_script_id())
        set_value('alternate_render_ids', source.get_script_layers())
        set_value('usuable', source.is_usable() and "yes" or "no")
        set_value('previewable', source.is_previewable() and "yes" or "no")
        set_value('cacheable', source.is_cacheable() and "yes" or "no")

        configuration_filename = os.path.join(directory, CONFIGURATION_FILE)
        files_to_keep = [CONFIGURATION_FILE]
        with open(configuration_filename, 'wb') as config_file:
            self._config.write(config_file)

        # Step 2, export parameters if any or delete the file.
        parameters = source.get_parameters_form()
        if parameters is not None:
            parameters_filename = os.path.join(directory, PARAMETERS_FILE)
            files_to_keep.append(PARAMETERS_FILE)
            with open(parameters_filename, 'w') as parameters_file:
                try:
                    parameters_file.write(formToXML(parameters) + os.linesep)
                except:
                    logger.error(
                        "failed to export parameters for code source %s",
                        source.getId())

        existing_mapping = {}
        for identifier, filename, installer in self._get_installables():
            existing_mapping[identifier] = filename

        # Step 2, export files.
        for identifier, content in source.objectItems():
            factory = EXPORTERS.get(content.meta_type, None)
            if factory is None:
                logger.info(
                    u"don't know how to export %s for code source %s",
                    content.meta_type, source.getId())
                continue
            exporter = factory(content)
            if identifier in existing_mapping:
                filename = existing_mapping[identifier]
            else:
                filename = exporter.get_path(identifier)
            try:
                exporter(os.path.join(directory, filename))
            except:
                logger.error(
                    "failed to export %s for code source %s",
                    content.meta_type, source.getId())
            files_to_keep.append(filename)

        # Step 3, purge files that were not recreated.
        for filename in os.listdir(directory):
            if filename not in files_to_keep:
                fullname = os.path.join(directory, filename)
                if os.path.isdir(fullname):
                    shutil.rmtree(fullname)
                else:
                    os.unlink(fullname)

        # Return the list of files
        return files_to_keep


class CodeSourceInstallable(CodeSourceExportable):
    grok.implements(ICodeSourceInstaller)

    def __init__(self, location, directory, extension=None):
        super(CodeSourceInstallable, self).__init__()
        self._config_filename = os.path.join(directory, CONFIGURATION_FILE)
        if os.path.isfile(self._config_filename):
            self._config.read(self._config_filename)
        self._directory = directory
        self._location = location
        self.extension = extension

    @property
    def identifier(self):
        if self._config.has_option('source', 'id'):
            return self._config.get('source', 'id')
        return os.path.basename(self._directory)

    @property
    def title(self):
        if self._config.has_option('source', 'title'):
            return self._config.get('source', 'title')
        return self.identifier

    @property
    def script_id(self):
        if self._config.has_option('source', 'render_id'):
            return self._config.get('source', 'render_id')
        return ''

    @property
    def description(self):
        if self._config.has_option('source', 'description'):
            return self._config.get('source', 'description')
        return None

    @property
    def location(self):
        return self._location

    def is_installed(self, folder):
        source = getattr(folder, self.identifier, None)
        return (ICodeSource.providedBy(source) and
                source.get_fs_location() == self._location)

    def install(self, folder):
        if self.is_installed(folder):
            return False
        if self.identifier in folder.objectIds():
            raise InstallationError(
                "There is already an object with the source identifier in the folder",
                self)

        factory = folder.manage_addProduct['SilvaExternalSources']
        factory.manage_addCodeSource(self.identifier, fs_location=self.location)
        source = folder._getOb(self.identifier)
        return self.update(source)

    def _get_installables(self):
        if not os.path.isdir(self._directory):
            return
        for filename in os.listdir(self._directory):
            if filename == CONFIGURATION_FILE or filename.startswith('.'):
                continue
            identifier, extension = os.path.splitext(filename)
            factory = INSTALLERS.get(extension, None)
            if factory is None:
                # Default to None, file default installer.
                factory = INSTALLERS[None]
            if factory.keep_extension:
                identifier = filename
            yield identifier, filename, factory()

    def update(self, source, purge=False):
        assert ICodeSource.providedBy(source)
        if source.get_fs_location() != self.location:
            raise InstallationError(
                u"Invalid source location", source.get_fs_location())
        if not self.validate():
            raise InstallationError(
                u"Source definition is incomplete", self)
        source.set_title(self.title)
        source.set_script_id(self.script_id)
        if self.description:
            source.set_description(self.description)
        if self._config.has_option('source', 'alternate_render_ids'):
            value = self._config.get('source', 'alternate_render_ids')
            try:
                source.set_script_layers(value)
            except ValueError:
                pass
        if self._config.has_option('source', 'cacheable'):
            value = self._config.getboolean('source', 'cacheable')
            source.set_cacheable(value)
        if self._config.has_option('source', 'previewable'):
            value = self._config.getboolean('source', 'previewable')
            source.set_previewable(value)
        if self._config.has_option('source', 'usable'):
            value = self._config.getboolean('source', 'usable')
            source.set_usable(value)

        installed = []
        for identifier, filename, installer in self._get_installables():
            if identifier in installed:
                raise InstallationError(u"Duplicate file", filename)
            if identifier in source.objectIds():
                source.manage_delObjects([identifier])
            installer(source, identifier,
                      os.path.join(self._directory, filename))
            installed.append(identifier)
        if purge:
            # Remove other files.
            source.manage_delObjects(
                list(set(source.objectIds()).difference(set(installed))))

        return True

    def export(self, source, directory=None):
        local = False
        if directory is None:
            if source.get_fs_location() != self.location:
                raise InstallationError(
                    u"Invalid source location", source.get_fs_location())
            directory = self._directory
            local = True

        source_files = super(CodeSourceInstallable, self).export(
            source, directory)
        if local:
            self._files = source_files


class CodeSourceService(SilvaService):
    meta_type = 'Silva Code Source Service'

    grok.implements(ICodeSourceService, ISilvaConfigurableService)
    grok.name('service_codesources')
    silvaconf.icon('www/codesource_service.png')

    security = ClassSecurityInfo()
    manage_options = (
        {'label': 'Available Code Sources',
         'action': 'manage_existing_codesources'},
        {'label': 'Install Code Sources',
         'action': 'manage_install_codesources'},
        {'label': 'External Sources Errors',
         'action': 'manage_sources_errors'}
        ) + SilvaService.manage_options

    # This is used a marker in to be backward compatible.
    _installed_sources = None

    security.declareProtected(
        'View management screens', 'find_installed_sources')

    def find_installed_sources(self):
        logger.info('search for code sources')
        self.clear_installed_sources()
        service = getUtility(IIntIds)
        for source in walk_silva_tree(self.get_root(), requires=ICodeSource):
            self._installed_sources.append(service.register(source))

    security.declareProtected(
        'Access contents information', 'get_installed_sources')

    def get_installed_sources(self):
        if self._installed_sources is not None:
            resolve = getUtility(IIntIds).getObject
            for source_id in self._installed_sources:
                try:
                    yield resolve(source_id)
                except KeyError:
                    pass

    security.declareProtected(
        'View management screens', 'clear_installed_sources')

    def clear_installed_sources(self):
        self._installed_sources = []

    security.declareProtected(
        'View management screens', 'get_installable_sources')

    def get_installable_sources(self, refresh=False):
        if not refresh and hasattr(self.aq_base,  '_v_installable_sources'):
            return self._v_installable_sources
        self._v_installable_sources = sources = []
        for entry_point in iter_entry_points(
                'Products.SilvaExternalSources.sources'):
            module = entry_point.load()
            directory = os.path.dirname(module.__file__)
            for source_identifier in os.listdir(directory):
                source_directory = os.path.join(directory, source_identifier)
                if not os.path.isdir(source_directory):
                    continue
                source_files = os.listdir(source_directory)
                if CONFIGURATION_FILE not in source_files:
                    continue
                source_location = (
                    entry_point.dist.project_name + ':' +
                    source_directory[len(entry_point.dist.location):])
                sources.append(CodeSourceInstallable(
                    source_location,
                    source_directory,
                    extension=entry_point.dist.project_name))
        return sources

    security.declareProtected(
        'View management screens', 'get_installable_source')

    def get_installable_source(self, identifier=None, location=None):
        if identifier is not None:
            test = lambda s: s.identifier == identifier
        elif location is not None:
            test = lambda s: s.location == location
        else:
            raise NotImplementedError
        for source in self.get_installable_sources():
            if test(source):
                yield source


InitializeClass(CodeSourceService)


@grok.subscribe(ICodeSource, IObjectAddedEvent)
def register_source(source, event):
    """Register newly created source to the service.
    """
    if (event.object is source and
            not IContainer.providedBy(event.newParent)):
        # The source is not added in a Silva Container so it won't be usable.
        return
    service = queryUtility(ICodeSourceService)
    if service is not None:
        source_id = getUtility(IIntIds).register(source)
        if service._installed_sources is None:
            service._installed_sources = []
        if source_id not in service._installed_sources:
            service._installed_sources.append(source_id)
            service._p_changed = True


@grok.subscribe(ICodeSource, IObjectWillBeRemovedEvent)
def unregister_source(source, event):
    """Remove deleted sources from the service.
    """
    if (event.object is source and
        event.newName is not None and
            IContainer.providedBy(event.newParent)):
        # We are just moving or renaming the source
        return
    service = queryUtility(ICodeSourceService)
    if service is not None and service._installed_sources is not None:
        source_id = getUtility(IIntIds).register(source)
        if source_id in service._installed_sources:
            service._installed_sources.remove(source_id)
            service._p_changed = True


OBJECT_ADDRESS = re.compile('0x([0-9a-f])*')


class SourcesError(object):
    """Describe a code source error.
    """

    def __init__(self, info, cleaned):
        self.info = info
        self._cleaned = cleaned
        self.count = 1
        self.when = datetime.now()

    def is_duplicate(self, other):
        if other == self._cleaned:
            self.count += 1
            return True
        return False

    def __getitem__(self, key):
        if key in self.__dict__:
            return self.__dict__[key]
        raise KeyError(key)


class SourcesErrorsReporter(grok.GlobalUtility):
    grok.implements(ISourceErrors)
    grok.provides(ISourceErrors)

    def __init__(self):
        self.clear()

    def report(self, info):
        logger.error(info)
        cleaned = OBJECT_ADDRESS.sub('0xXXXXXXX', info)
        for error in self.__errors:
            if error.is_duplicate(cleaned):
                return
        self.__errors.append(SourcesError(info, cleaned))

    def __len__(self):
        return len(self.__errors)

    def fetch(self):
        return list(reversed(self.__errors))

    def clear(self):
        self.__errors = collections.deque([], 25)


class ExistingCodeSourcesMixin(object):

    def _get_source_name(self, source):
        return source.getId()

    def update(self, find=False, below=None, child=False,
               update=False, bind=False, sources=None):
        self.success = []
        self.errors = []
        if find:
            self.context.find_installed_sources()
            self.success.append(_(u"List of sources refreshed."))

        self.sources = []
        self.include_child = bool(child)
        if below:
            below = below.strip()
            if not below.endswith('/'):
                below += '/'
        for source in self.context.get_installed_sources():
            path = '/'.join(source.getPhysicalPath())
            if (below and (not path.startswith(below) or
                           (not child and '/' in path[len(below):]))):
                continue
            if isinstance(source, Broken):
                self.sources.append(
                    {'id': source.getId(),
                     'problems': [_(u'Filesystem code is missing')],
                     'title': _(u'Corresponding Source implementation is missing'),
                     'path': path,
                     'url': None})
                self.errors.append(_(
                    '${source}: Source have problems.',
                    mapping=dict(source=self._get_source_name(source))))
                continue
            status = None
            if ICodeSource.providedBy(source):
                if update and (sources is None or source.getId() in sources):
                    installable = source._get_installable()
                    name = self._get_source_name(source)
                    if (installable is not None and
                            os.path.isdir(installable._directory)):
                        installable.update(source, True)
                        status = _('${name}: Source updated.',
                                   mapping=dict(name=name))
                    else:
                        self.sources.append(
                            {'id': source.getId(),
                             'problems': [_(u'Filesystem code have been deleted')],
                             'title': source.get_title(),
                             'path':  path,
                             'url': source.absolute_url()})
                        self.errors.append(_('${name}: Source have problems.',
                                             mapping=dict(name=name)))
                        continue
                elif bind and not source.get_fs_location():
                    candidates = source.manage_getFileSystemLocations()
                    if len(candidates) == 1:
                        source._fs_location = candidates[0]
                        status = _(
                            '${source}: Source associated with ${location}.',
                            mapping=dict(source=self._get_source_name(source),
                                         location=candidates[0]))
            if status:
                self.success.append(status)
            self.sources.append({'id': source.getId(),
                                 'problems': source.test_source(),
                                 'title': source.get_title(),
                                 'path': path,
                                 'url': source.absolute_url()})
        self.sources.sort(key=operator.itemgetter('title'))
        if below:
            self.filter = below.rstrip('/')
        else:
            self.filter = '/'.join(aq_parent(self.context).getPhysicalPath())


class ManageExistingCodeSources(ExistingCodeSourcesMixin, silvaviews.ZMIView):
    grok.name('manage_existing_codesources')
    grok.context(CodeSourceService)


class ConfigureExisting(ExistingCodeSourcesMixin, rest.FormWithTemplateREST):
    grok.adapts(rest.Screen, CodeSourceService)
    grok.name('admin-existing')
    grok.require('zope2.ViewManagementScreens')

    def get_menu_title(self):
        return _('Available code sources')

    def get_menu_parent(self):
        parent = super(ConfigureExisting, self).get_menu_parent()
        parent['screen'] = 'admin'
        return parent

    def _get_source_name(self, source):
        return source.get_title()

    def update(self):
        sources = self.request.form.get('sources')
        if sources is not None and not isinstance(sources, list):
            sources = [sources]
        super(ConfigureExisting, self).update(
            find='find' in self.request.form,
            update='update' in self.request.form,
            sources=sources)
        if self.success or self.errors:
            service = getUtility(IMessageService)
            for status in self.success:
                service.send(status, self.request, namespace='feedback')
            for status in self.errors:
                service.send(status, self.request, namespace='error')


class ConfigureExistingMenu(menu.MenuItem):
    grok.adapts(menu.ContentMenu, CodeSourceService)
    grok.order(20)
    name = _('Available')
    screen = ConfigureExisting


class InstallCodeSourcesMixin(object):
    only_uninstalled = False

    def _get_source_name(self, installable):
        return installable.identifier

    def update(self, install=False, refresh=False, locations=[]):
        self.success = []
        self.errors = []
        if install:
            if not isinstance(locations, list):
                locations = [locations]
            for location in locations:
                candidates = list(self.context.get_installable_source(
                    location=location))
                if len(candidates) != 1:
                    self.errors.append(
                        _('${location}: Source was not found and could not be installed.',
                          mapping=dict(source=location)))
                else:
                    installable = candidates[0]
                    name = self._get_source_name(installable)
                    try:
                        if installable.install(self.context.get_root()):
                            self.success.append(
                                _('${name}: Source was installed.',
                                  mapping=dict(name=name)))
                        else:
                            self.errors.append(
                                _('${name}: Source is already installed.',
                                  mapping=dict(name=name)))
                    except InstallationError as error:
                        self.errors.append(
                            _('${name}: Error during the installation: ${error}.',
                              mapping=dict(name=name,
                                           error=error.args[0])))

        self.extensions = []
        self.sources = 0
        extensions = {}
        for source in self.context.get_installable_sources(refresh=refresh):
            if self.only_uninstalled and source.is_installed(self.context):
                continue
            sources = extensions.setdefault(source.extension, [])
            sources.append(source)
            self.sources += 1
        for name, sources in extensions.items():
            sources.sort(key=operator.attrgetter('title'))
            if name is None:
                self.extensions.append({
                    'title': _('Default code sources'),
                    'id': '0',
                    'description': '',
                    'sources': sources})
                continue
            identifier = str(name).encode('base64').strip().rstrip('=')
            extension = extensionRegistry.get_extension(name)
            if extension is None:
                self.extensions.append({
                    'title': name,
                    'id': identifier,
                    'description': '',
                    'sources': sources})
                continue
            self.extensions.append({
                'title': extension.title,
                'id': identifier,
                'description': extension.description,
                'sources': sources})
        self.extensions.sort(key=operator.itemgetter('title'))


class ManageInstallCodeSources(InstallCodeSourcesMixin, silvaviews.ZMIView):
    grok.name('manage_install_codesources')
    grok.context(CodeSourceService)

    def update(self, install=False, refresh=False, locations=[]):
        super(ManageInstallCodeSources, self).update(
            install=install, refresh=refresh, locations=locations)
        need(jquery)


class ConfigureInstall(InstallCodeSourcesMixin, rest.FormWithTemplateREST):
    grok.adapts(rest.Screen, CodeSourceService)
    grok.name('admin')
    grok.require('zope2.ViewManagementScreens')

    only_uninstalled = True

    def get_menu_title(self):
        return _('Install code sources')

    def get_menu_parent(self):
        parent = super(ConfigureInstall, self).get_menu_parent()
        parent['screen'] = 'admin'
        return parent

    def _get_source_name(self, installable):
        return installable.title

    def update(self, install=False, refresh=False, locations=[]):
        super(ConfigureInstall, self).update(
            install='install' in self.request.form,
            refresh='refresh' in self.request.form,
            locations=self.request.form.get('locations', []))
        if self.success or self.errors:
            service = getUtility(IMessageService)
            for status in self.success:
                service.send(status, self.request, namespace='feedback')
            for status in self.errors:
                service.send(status, self.request, namespace='error')


class ConfigureInstallMenu(menu.MenuItem):
    grok.adapts(menu.ContentMenu, CodeSourceService)
    grok.order(10)
    name = _('Install')
    screen = ConfigureInstall


class ManageSourcesErrors(silvaviews.ZMIView):
    grok.name('manage_sources_errors')
    grok.context(CodeSourceService)

    def update(self):
        errors = getUtility(ISourceErrors)
        if 'clear' in self.request.form:
            errors.clear()
        self.errors = errors.fetch()


def check_extension(name):
    if ':' in name:
        name, _ = name.split(':', 1)
    return name in pkg_resources.working_set.by_key


class IExportCodeSourcesFields(Interface):

    extension_name = schema.TextLine(
        title=u'Extension name',
        description=_(
            u"Enter the extension name and the entry point name, seperated by "
            u"a :. The entry point name defaults to defaults if it is not "
            u"specified."),
        constraint=check_extension,
        required=True)
    recursive = schema.Bool(
        title=u'Recursive export ?',
        default=False,
        required=False)


class ManageExportCodeSources(silvaforms.ZMIForm):
    grok.name('manage_export_codesources')
    grok.context(IContainer)

    label = u'Mass export of codesources'
    description = u'Export every codesources located in this container and ' \
        u'optionally in sub-containers.'
    fields = silvaforms.Fields(IExportCodeSourcesFields)

    @silvaforms.action('Export')
    def export(self):
        values, errors = self.extractData()
        if errors:
            return silvaforms.FAILURE
        exported = []
        extension_name = values['extension_name']
        if ':' in extension_name:
            extension_name, entry_name = extension_name.split(':', 1)
        extension = pkg_resources.working_set.by_key[extension_name]
        directory = os.path.dirname(extension.load_entry_point(
            'Products.SilvaExternalSources.sources', entry_name).__file__)

        if values['recursive']:
            sources = walk_silva_tree(self.context, requires=ICodeSource)
        else:
            sources = self.context.objectValues('Silva Code Source')

        for source in sources:
            if source.meta_type != 'Silva Code Source':
                continue
            identifier = source.getId()
            target = os.path.join(directory, identifier)
            location = (
                extension.project_name + ':' +
                target[len(extension.location):])

            if source.get_fs_location() not in (None, location):
                continue
            if not os.path.exists(target):
                os.makedirs(target)
            source._fs_location = location
            installable = CodeSourceInstallable(location, target)
            installable.export(source)
            exported.append(location)
        if exported:
            self.status = 'Exported: {0}.'.format(', '.join(exported))
        else:
            self.status = 'Nothing exported.'
        return silvaforms.SUCCESS



##parameters=model, version, ref, controller, autoplay, width='', height=''
# Embeds movies using the Windows Media Player. This Code Source
# was developped to embed *.avi files - therefore it uses the
# clsid of the class ID of the Windows Media Player 6 
# (clsid:05589FA1-C356-11CE-BF01-00AA0055595A) instead of the newer
# ID of the Player versions 7 - 10
# (clsid:6BF52A52-394A-11D3-B153-00C04F79FAA6), which did not work with
# *.avi files. Furthermore it includes 'type="application/x-mplayer2"' in
# the embed tag (the 'video/x-ms-wmv' type would work too, but this Code
# Source explicitely calls for the Windows Media Player). The Windows
# Media Player Plug-in DLL in Firefox does not listen to the extension
# *.avi or its correct MIME-type, 'video/x-msvideo' (or 'video/avi').


video_url = ref.absolute_url()
if width == '':
    width_attr = ''
else:
    width_attr = 'width="%s" ' % width

if height == '':
    height_attr = ''
else:
    height_attr = 'height="%s" ' % height

if controller == 'true':
    embed_controller = '1'
else:
    embed_controller = '0'

if autoplay == 'true':
    embed_autoplay = '1'
else:
    embed_autoplay = '0'

return """
<p class="p">
<object %(width_attr)s%(height_attr)s classid="clsid:05589FA1-C356-11CE-BF01-00AA0055595A">
  <param name="src" value="%(video_url)s" />
  <param name="autostart" value="%(embed_autoplay)s" />
  <param name="showcontrols" value="%(embed_controller)s" />
  <embed %(width_attr)s%(height_attr)ssrc="%(video_url)s"
    type="application/x-mplayer2" autostart="%(autoplay)s" showcontrols="%(controller)s" />
</object>
</p>
""" % {"width_attr":width_attr, "height_attr":height_attr, "video_url":video_url,
       "embed_autoplay":embed_autoplay, "embed_controller":embed_controller, "autoplay":autoplay, "controller":controller}


#original
#return """
#<p class="p">
#<object %s%s classid="clsid:05589FA1-C356-11CE-BF01-00AA0055595A">
  #<param name="src" value="%s" />
  #<param name="autostart" value="%s" />
  #<param name="showcontrols" value="%s" />
  #<embed %s%ssrc="%s"
    #type="application/x-mplayer2" autostart="%s" showcontrols="%s" />
#</object>
#</p>
#""" % (width_attr, height_attr, video_url, embed_autoplay, embed_controller,
       #width_attr, height_attr, video_url, autoplay, controller)





# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt
# package

from silva.core.xml import registerNamespace

NS_SOURCE_URI = "http://infrae.com/namespace/silva-external-sources"
registerNamespace('silva-external-sources', NS_SOURCE_URI)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from random import choice
from AccessControl import ModuleSecurityInfo

module_security = ModuleSecurityInfo(
    'Products.SilvaExternalSources.codesources.make_HTML_token')

chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz-0123456789'
length = 12

module_security.declarePublic('make_token')
def make_token():
    return ''.join(choice(chars) for x in xrange(length))



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from zope.publisher.browser import TestRequest
from zope.interface.verify import verifyObject
from zeam.component import getWrapper

from Products.Silva.tests.test_xml_import import SilvaXMLTestCase

from silva.app.document.interfaces import IDocument
from silva.core.references.reference import ReferenceSet

from ..errors import SourceMissingError
from ..interfaces import IExternalSourceManager
from ..interfaces import ISourceAsset, ISourceAssetVersion
from ..testing import FunctionalLayer


class CodeSourceDocumentImportTestCase(SilvaXMLTestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def test_document_with_source(self):
        """Import a document that uses a source.
        """
        importer = self.assertImportFile(
            'test_import_source.silvaxml',
            ['/root/example'])
        self.assertEqual(importer.getProblems(), [])

        document = self.root.example
        self.assertTrue(verifyObject(IDocument, document))
        self.assertEqual(document.get_viewable(), None)
        self.assertNotEqual(document.get_editable(), None)

        version = document.get_editable()
        sources = getWrapper(version, IExternalSourceManager)
        keys = list(sources.all())
        self.assertEqual(len(keys), 1)
        parameters, source = sources.get_parameters(instance=keys[0])
        self.assertEqual(parameters.get_source_identifier(), 'cs_citation')
        self.assertEqual(source.id, 'cs_citation')
        self.assertEqual(parameters.citation, u"héhé l'aime le quéqué")
        self.assertEqual(parameters.author, u'ouam')
        self.assertEqual(parameters.source, u'wikipedia')

    def test_document_with_missing_source(self):
        """Import a document that uses a source that is missing on the
        system.

        The document is imported, but not the source.
        """
        importer = self.assertImportFile(
            'test_import_source_missing.silvaxml',
            ['/root/example'])

        document = self.root.example
        self.assertTrue(verifyObject(IDocument, document))
        self.assertEqual(document.get_viewable(), None)
        self.assertNotEqual(document.get_editable(), None)

        version = document.get_editable()
        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 0)
        self.assertEqual(
            importer.getProblems(),
            [(u'Broken source in import: '
              u'External Source cs_ultimate is not available.', version)])


class SourceAssetImportTestCase(SilvaXMLTestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def test_source_asset(self):
        importer = self.assertImportFile(
            'test_import_source_asset.silvaxml',
            ['/root/folder',
             '/root/folder/asset'])
        self.assertEqual(importer.getProblems(), [])

        asset = self.root.folder.asset
        self.assertTrue(verifyObject(ISourceAsset, asset))
        version = asset.get_editable()
        self.assertTrue(verifyObject(ISourceAssetVersion, version))
        source = version.get_source()
        self.assertEquals('cs_toc', source.getId())
        params = version.get_parameters()
        self.assertIn(self.root.folder, ReferenceSet(version, params.paths))
        self.assertEquals(set(['Silva Folder']), set(params.toc_types))

    def test_source_asset_missing_source(self):
        """Import a source asset content that uses a unexisting
        source.
        """
        importer = self.assertImportFile(
            'test_import_source_asset_missing.silvaxml',
            ['/root/folder',
             '/root/folder/asset'])

        asset = self.root.folder.asset
        self.assertTrue(verifyObject(ISourceAsset, asset))
        version = asset.get_editable()
        self.assertTrue(verifyObject(ISourceAssetVersion, version))
        with self.assertRaises(SourceMissingError):
            version.get_source()

        self.assertEqual(
            importer.getProblems(),
            [(u'Broken source in import: '
              u'External Source cs_missing is not available.', version)])

    def test_source_asset_invalid_parameters(self):
        """Import a source asset content with invalid parameters.
        """
        importer = self.assertImportFile(
            'test_import_source_asset_invalid_parameters.silvaxml',
            ['/root/folder',
             '/root/folder/asset'])

        asset = self.root.folder.asset
        self.assertTrue(verifyObject(ISourceAsset, asset))
        version = asset.get_editable()
        self.assertTrue(verifyObject(ISourceAssetVersion, version))
        source = version.get_source()
        self.assertEquals(source.getId(), 'cs_toc')
        params = version.get_parameters()
        self.assertEqual(set(params.toc_types), set(['Silva Folder']))
        self.assertEqual(
            importer.getProblems(),
            [("Error in field 'paths': Could not resolve imported path non/existant.", version),
             ("Error in field 'paths': Broken reference.", version)])

    def test_source_asset_unknown_parameters(self):
        """This export contains extra unknown parameters.
        """
        importer = self.assertImportFile(
            'test_import_source_asset_unknown_parameters.silvaxml',
            ['/root/folder',
             '/root/folder/asset'])
        self.assertEqual(importer.getProblems(), [])

        asset = self.root.folder.asset
        self.assertTrue(verifyObject(ISourceAsset, asset))
        version = asset.get_editable()
        self.assertTrue(verifyObject(ISourceAssetVersion, version))
        source = version.get_source()
        self.assertEqual(source.getId(), 'cs_toc')
        params = version.get_parameters()
        self.assertIn(self.root.folder, ReferenceSet(version, params.paths))
        self.assertEqual(set(params.toc_types), set(['Silva Folder']))
        self.assertIs(getattr(params, 'space_depth', None), None)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CodeSourceDocumentImportTestCase))
    suite.addTest(unittest.makeSuite(SourceAssetImportTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import logging
import sys
import uuid

from five import grok
from zope.component import getUtility
from zope.annotation.interfaces import IAnnotatable, IAnnotations
import persistent

# Zope
from AccessControl import ClassSecurityInfo
from AccessControl.security import checkPermission
from App.class_init import InitializeClass
from zExceptions.ExceptionFormatter import format_exception
import Acquisition

# Silva
from Products.Silva import SilvaPermissions as permissions

from silva.translations import translate as _
from zeam.component import component
from zeam.form import silva as silvaforms
from zeam.form.ztk.widgets import TextField

from . import errors as error
from .interfaces import IExternalSource, IEditableExternalSource
from .interfaces import IExternalSourceController
from .interfaces import IExternalSourceManager, IExternalSourceInstance
from .interfaces import ISourceErrors
from .interfaces import availableSources # BB

logger = logging.getLogger('silva.externalsources')
SETTINGS_PERMISSION = 'silva.ManageSilvaContentSettings'

# BBB
def getSourceForId(context, identifier):
    """ Look for an Source with given id. Mimic normal aqcuisition,
    but skip objects which have given id but do not implement the
    ExternalSource interface.
    """
    nearest = getattr(context, identifier, None)
    if IExternalSource.providedBy(nearest):
        return nearest
    return None


class ExternalSource(Acquisition.Implicit):
    grok.implements(IExternalSource)
    security = ClassSecurityInfo()
    parameters = None

    _data_encoding = 'UTF-8'
    _description = ''
    _is_cacheable = False
    _is_previewable = True
    _is_usable = True

    # ACCESSORS

    security.declareProtected(
        permissions.AccessContentsInformation, 'get_parameters_form')
    def get_parameters_form(self):
        """ Return the parameters fields.
        """
        return self.parameters

    security.declareProtected(
        permissions.AccessContentsInformation, 'to_html')
    def to_html(self, content, request, **parameters):
        """ Render the HTML for inclusion in the rendered Silva HTML.
        """
        raise NotImplementedError

    security.declareProtected(
        permissions.AccessContentsInformation, 'is_usable')
    def is_usable(self):
        """ Return true if the external source must be usable.
        """
        return self._is_usable

    security.declareProtected(
        permissions.AccessContentsInformation, 'is_cacheable')
    def is_cacheable(self, **parameters):
        """ Specify the cacheability.
        """
        return self._is_cacheable

    security.declareProtected(
        permissions.AccessContentsInformation, 'is_previewable')
    def is_previewable(self, **parameters):
        """ Specify the previewability (in kupu) of the source
        """
        if not hasattr(self, '_is_previewable'):
            self._is_previewable = True
        return self._is_previewable

    security.declareProtected(
        permissions.AccessContentsInformation, 'get_data_encoding')
    def get_data_encoding(self):
        """ Specify the encoding of source's data.
        """
        return self._data_encoding

    security.declareProtected(
        permissions.AccessContentsInformation, 'get_description')
    def get_description(self):
        """ Specify the use of this source.
        """
        return self._description

    security.declareProtected(
        permissions.AccessContentsInformation, 'get_title')
    def get_title (self):
        return self.title or self.getId()

    security.declareProtected(
        permissions.AccessContentsInformation, 'get_icon')
    def get_icon(self):
        return None


InitializeClass(ExternalSource)


class EditableExternalSource(ExternalSource):
    grok.implements(IEditableExternalSource)
    security = ClassSecurityInfo()

    security.declareProtected(
        permissions.ViewManagementScreens, 'set_form')
    def set_form(self, form):
        """Set Formulator parameters form
        """
        self.parameters = form

    security.declareProtected(
        permissions.ViewManagementScreens, 'set_data_encoding')
    def set_data_encoding(self, encoding):
        """Set encoding of data.
        """
        self._data_encoding = encoding

    security.declareProtected(
        permissions.ViewManagementScreens, 'set_title')
    def set_title(self, title):
        """Set title
        """
        self.title = title

    security.declareProtected(
        permissions.ViewManagementScreens, 'set_description')
    def set_description(self, description):
        """Set description of external source's use.
        """
        self._description = description

    security.declareProtected(
        permissions.ViewManagementScreens, 'set_cacheable')
    def set_cacheable(self, cacheable):
        """Set cacheablility of source.
        """
        self._is_cacheable = cacheable

    security.declareProtected(
        permissions.ViewManagementScreens, 'set_previewable')
    def set_previewable(self, previewable):
        """Set previewablility of source.
        """
        self._is_previewable = bool(previewable)

    security.declareProtected(
        permissions.ViewManagementScreens, 'set_usable')
    def set_usable(self, usable):
        """Set usability.
        """
        self._is_usable = bool(usable)

InitializeClass(EditableExternalSource)


class ExternalSourceParameters(persistent.Persistent):
    grok.implements(IExternalSourceInstance)

    __source_template = None

    def __init__(self, parameter_identifier, source_identifier):
        self.__source_identifier = source_identifier
        self.__parameter_identifier = parameter_identifier

    def get_parameter_identifier(self):
        return self.__parameter_identifier

    def get_source_identifier(self):
        return self.__source_identifier

    def get_source_template(self):
        # A source template is an HTML chunk that can be used to warp
        # around the result of an external source.
        return self.__source_template

    def set_source_template(self, template):
        self.__source_template = template


@component(IAnnotatable, provides=IExternalSourceManager)
class ExternalSourceManager(object):
    grok.implements(IExternalSourceManager)
    KEY = 'Products.SilvaExternalSources'

    def __init__(self, context):
        self.context = context
        self.annotations = IAnnotations(self.context)
        self.sources = self.annotations.get(self.KEY)

    def editable_sources(self):
        # Add the annotations if missing, and return the sources.
        # This is not done in the __init__ because it can be built
        # during tranversing (read-only).
        if self.sources is None:
            self.sources = self.annotations[self.KEY] = \
                persistent.mapping.PersistentMapping()
        return self.sources

    def new(self, source):
        assert IExternalSource.providedBy(source)
        identifier = str(uuid.uuid1())
        parameters = ExternalSourceParameters(identifier, source.id)
        self.editable_sources()[identifier] = parameters
        return parameters

    def remove(self, identifier):
        del self.editable_sources()[identifier]

    def all(self):
        if self.sources is not None:
            return set(self.sources.keys())
        return []

    def get_parameters(self, instance=None, name=None):
        parameters = None
        if instance is not None:
            if self.sources is not None:
                parameters = self.sources.get(instance)
            if parameters is None:
                raise error.ParametersMissingError(instance)
            name = parameters.get_source_identifier()
        if name is None:
            raise error.SourceMissingError('unknow')
        source = getattr(self.context, name, None)
        if source is not None and IExternalSource.providedBy(source):
            return parameters, source
        if parameters is not None:
            # XXX Return a stub here not to loose source name.
            return parameters, None
        raise error.SourceMissingError(name)

    def __call__(self, request, instance=None, name=None):
        parameters, source = self.get_parameters(instance=instance, name=name)
        return ExternalSourceController(source, self, request, parameters)


class InvalidSourceTemplate(ValueError):

    def doc(self):
        return _(u'The source template marker <!-- source output --> '
                 u'is missing.')


def validateSourceTemplate(value):
    if '<!-- source output -->' not in value:
        raise InvalidSourceTemplate()
    return True


class ExternalSourceController(silvaforms.FormData):
    grok.implements(IExternalSourceController)
    security = ClassSecurityInfo()

    actions = silvaforms.Actions()
    parameterFields = silvaforms.Fields()
    settingFields = silvaforms.Fields()
    ignoreRequest = True
    ignoreContent = False

    def __init__(self, source, manager, request, instance):
        dataManager = [
            (None,
             lambda content: silvaforms.FieldValueDataManager(self, content))]
        self.manager = manager
        self.source = source
        self.__parent__ = manager.context # Enable security checks.
        if source is not None:
            fields = source.get_parameters_form()
            if fields is not None:
                self.parameterFields = silvaforms.Fields(fields)
            # This doesn't work because authentication is not done
            # when __init__ is called.
            if checkPermission(SETTINGS_PERMISSION, manager.context):
                self.settingFields += TextField(
                    identifier="source_template",
                    title="Source template",
                    description="<!-- source output --> will be replaced by the HTML code generated by the source.",
                    constrainValue=validateSourceTemplate,
                    required=False)
                dataManager.append(
                    ('source_template', silvaforms.SilvaDataManager))
        self.dataManager = silvaforms.MultiDataManagerFactory(dataManager)
        super(ExternalSourceController, self).__init__(
            manager.context, request, instance)

    def getId(self):
        content = self.getContent()
        if content is not None:
            return content.get_parameter_identifier()
        return None

    security.declareProtected(
        permissions.AccessContentsInformation, 'getSourceId')
    def getSourceId(self):
        return self.source.id

    def extractData(self, fields=None):
        if fields is None:
            fields = self.parameterFields + self.settingFields
        return super(ExternalSourceController, self).extractData(fields)

    def editable(self):
        return len(self.parameterFields) != 0

    @property
    def label(self):
        if self.source is not None:
            return self.source.get_title()
        return _(u"Broken source")

    @property
    def description(self):
        if self.source is not None:
            return self.source.get_description()
        return u''

    @property
    def fields(self):
        return self.parameterFields

    def indexes(self):
        # Return index entries for Silva Indexer.
        return []

    def fulltext(self):
        # Return fulltext for the catalog
        return []

    def new(self):
        assert self.source is not None, u'Cannot create broken source'
        assert self.getContent() is None, u'Cannot override existing source'
        self.setContentData(self.manager.new(self.source))
        return self.getId()

    def copy(self, destination):
        assert self.source is not None, u'Cannot copy broken source'
        assert self.getSourceId() == destination.getSourceId()
        source = self.getContentData()
        target = destination.getContentData()
        for field in self.parameterFields:
            try:
                target.set(field.identifier, source.get(field.identifier))
            except KeyError:
                pass

    @silvaforms.action(_(u"Create"))
    def create(self):
        self.new()
        return self.save()

    @silvaforms.action(_(u"Save"))
    def save(self):
        assert self.getContent() is not None, u'Cannot save to missing source'
        if self.source is None:
            raise error.SourceMissingError('unknow')
        data, errors = self.extractData()
        if errors:
            return silvaforms.FAILURE
        manager = self.getContentData()
        for field in self.parameterFields + self.settingFields:
            value = data.getWithDefault(field.identifier)
            if value is not silvaforms.NO_CHANGE:
                manager.set(field.identifier, value)
        return silvaforms.SUCCESS

    @silvaforms.action(_(u"Remove"))
    def remove(self):
        assert self.getContent() is not None, u'Cannot remove missing source'
        manager = self.getContentData()
        identifier = self.getId()
        for field in self.parameterFields:
            manager.delete(field.identifier)
        self.manager.remove(identifier)
        return silvaforms.SUCCESS

    def fieldWidgets(self, ignoreRequest=False, ignoreContent=True, display=False):
        if display:
            self.mode = silvaforms.DISPLAY
            self.ignoreRequest = True
            self.ignoreContent = False
        else:
            self.mode = silvaforms.INPUT
            self.ignoreRequest = ignoreRequest
            self.ignoreContent = ignoreContent
        widgets = silvaforms.Widgets(form=self, request=self.request)
        widgets.extend(self.parameterFields)
        if self.settingFields and not display:
            widgets.extend(self.settingFields)
        widgets.update()
        return widgets

    security.declareProtected(
        permissions.AccessContentsInformation, 'render')
    def render(self, view=False, preview=False):
        if self.source is None:
            raise error.SourceMissingError('unknow')
        if preview and not self.source.is_previewable():
            raise error.SourcePreviewError(self)
        values = {}
        template = None
        if self.parameterFields:
            if not self.ignoreRequest:
                values, errors = self.extractData(self.parameterFields)
                if errors:
                    raise error.ParametersError(errors)
            elif not self.ignoreContent and self.getContent() is not None :
                template = self.getContent().get_source_template()
                manager = self.getContentData()
                for field in self.parameterFields:
                    try:
                        value = manager.get(field.identifier)
                    except KeyError:
                        value = field.getDefaultValue(self)
                    values[field.identifier] = value
            else:
                raise error.ParametersError()

        try:
            html = self.source.to_html(self.context, self.request, **values)
        except:
            info = u''.join(format_exception(*sys.exc_info()))
            getUtility(ISourceErrors).report(info)
            raise error.SourceRenderingError(self, info)
        if not preview and template:
            html = template.replace('<!-- source output -->', html)
        return html

InitializeClass(ExternalSourceController)



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from operator import itemgetter
import cgi
import io
import os
import shutil
import tempfile
import zipfile

# Zope
from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.Formulator.Form import ZMIForm
from Products.Silva import SilvaPermissions
from Products.Silva.helpers import add_and_edit

from five import grok
from zope.component import queryUtility
from zope.publisher.interfaces.browser import IBrowserSkinType

from silva.core.interfaces.content import IVersion
from silva.core.services.base import ZMIObject
from silva.core import conf as silvaconf

from .interfaces import ICodeSource, ICodeSourceService
from .ExternalSource import EditableExternalSource
from .CodeSourceService import CodeSourceExportable


class CodeSourceErrorSupplement(object):
    """Add more information about an error happening during the
    rendering of a code source.
    """

    def __init__(self, source, parameters):
        self.source = source
        self.parameters = parameters

    def getInfo(self, as_html=0):
        info = list()
        info.append((u'Source path', '/'.join(self.source.getPhysicalPath())))
        if 'model' in self.parameters:
            document = self.parameters['model']
            info.append((u'Document type', document.meta_type))
            info.append((u'Document path', '/'.join(document.getPhysicalPath())))
        if 'version' in self.parameters:
            version = self.parameters['version']
            info.append((u'Document version', version.getId()))
        for name, value in self.parameters.items():
            if name not in ('model', 'version'):
                info.append((u'Parameter %s' % name, repr(value)))

        if not as_html:
            return '   - ' + '\n   - '.join(map(lambda x: '%s: %s' % x, info))

        return u'<p>Extra information:<br /><li>%s</li></p>' % ''.join(map(
                lambda x: u'<li><b>%s</b>: %s</li>' % (
                    cgi.escape(str(x[0])), cgi.escape(str(x[1]))),
                info))


class CodeSource(EditableExternalSource, Folder, ZMIObject):
    grok.implements(ICodeSource)
    # register icon and factories
    silvaconf.icon('www/codesource.png')
    silvaconf.factory('manage_addCodeSourceForm')
    silvaconf.factory('manage_addCodeSource')
    silvaconf.zmi_addable(True)

    meta_type = "Silva Code Source"
    security = ClassSecurityInfo()

    _data_encoding = 'UTF-8'
    _fs_location = None
    _script_layers = []

    # ZMI Tabs
    manage_options = (
        {'label':'Edit', 'action':'editCodeSource'},
        {'label':'Parameters', 'action':'parameters/manage_main'},
        ) + Folder.manage_options
    management_page_charset = 'utf-8'

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'editCodeSource')
    editCodeSource = PageTemplateFile(
        'www/codeSourceEdit', globals(),  __name__='editCodeSource')


    def __init__(self, id, script_id=None, fs_location=None):
        super(CodeSource, self).__init__(id)
        self._script_id = script_id
        self._fs_location = fs_location

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'test_source')
    def test_source(self):
        # return a list of problems or None
        errors = []
        # in real life the parent of the form is the document. We try
        # to do the same here.
        root = self.get_root()
        if root.get_default():
            root = root.get_default()
        if self.parameters is not None:
            try:
                self.parameters.test_form(
                    context=root,
                    bad_fields=['context', 'content', 'model', 'script'])
            except ValueError as error:
                errors.extend(error.args)
        if not self.title:
            errors.append(u'Missing required source title.')
        if not self._script_id:
            errors.append(u'Missing required renderer id.')
        else:
            ids = self.objectIds()
            scripts = [self._script_id] + map(
                itemgetter(0), self._script_layers)
            for script_id in scripts:
                if script_id not in ids:
                    errors.append(
                        u'Missing renderer %s. Please a script or template with this id.' % (
                            script_id))
        if errors:
            return errors
        return None

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_icon')
    def get_icon(self):
        return self._getOb('icon.png', None)

    # ACCESSORS
    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_script_id')
    def get_script_id(self):
        return self._script_id

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_script_layers')
    def get_script_layers(self):
        result = []
        skin = grok.skin.bind(default=lambda l: l.__identifier__)
        for script_id, layer in self._script_layers:
            result.append(":".join((script_id, skin.get(layer))))
        return '\n'.join(result)

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'get_fs_location')
    def get_fs_location(self):
        return self._fs_location

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'to_html')
    def to_html(self, content, request, **parameters):
        """Render HTML for code source
        """
        script = None
        if self._script_layers:
            # If there are script_layer, check them first.
            for script_id, layer in self._script_layers:
                if layer.providedBy(request):
                    break
            else:
                # No matching layer, default.
                script_id = self._script_id
        else:
            # No script_layer, default one.
            script_id = self._script_id
        if script_id is not None:
            script = self._getOb(script_id, None)
        if script_id is None or script is None:
            # Missing script
            return None
        parameters['REQUEST'] = request
        if IVersion.providedBy(content):
            parameters['version'] = content
            parameters['model'] = content.get_silva_object()
        else:
            parameters['version'] = None
            parameters['model'] = content
        __traceback_supplement__ = (CodeSourceErrorSupplement, self, parameters)
        result = script(**parameters)
        if isinstance(result,  unicode):
            return result
        return unicode(result, self.get_data_encoding(), 'replace')

    # MANAGERS

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'set_script_id')
    def set_script_id(self, script_id):
        self._script_id = script_id

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'set_script_layers')
    def set_script_layers(self, script_layers):
        found = []
        for lineno, line in enumerate(script_layers.strip().splitlines()):
            entries = line.strip().split(':', 1)
            if len(entries) != 2:
                raise ValueError(
                    u'Invalid script layers: invalid form on line %d' % (
                        lineno))
            script_id, layer_identifier = entries
            layer = queryUtility(IBrowserSkinType, name=layer_identifier)
            if layer is None:
                raise ValueError(
                    u'Invalid script layer: layer %s not found on line %d' % (
                        layer_identifier, lineno))
            found.append((script_id, layer))
        self._script_layers = found

    def _get_installable(self, location=None):
        """Return the installable source associated with this code
        source.
        """
        if location is None:
            location = self.get_fs_location()
            if location is None:
                return None
        service = queryUtility(ICodeSourceService)
        if service is None:
            # XXX pre-migration Silva 3.0
            service = self.service_codesources
        candidates = list(service.get_installable_source(location=location))
        if len(candidates) == 1:
            return candidates[0]
        return None

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'manage_getFileSystemLocations')
    def manage_getFileSystemLocations(self):
        service = queryUtility(ICodeSourceService)
        if service is None:
            # XXX pre-migration Silva 3.0
            service = self.service_codesources
        return map(
            lambda source: source.location,
            service.get_installable_source(identifier=self.id))

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'manage_updateCodeSource')
    def manage_updateCodeSource(self, purge=False, REQUEST=None):
        """Update a code source from the filesystem.
        """
        installable = self._get_installable()
        if (installable is None or
                not os.path.isdir(installable._directory)):
            if REQUEST is not None:
                return self.editCodeSource(
                    manage_tabs_message=\
                        'Couldn\'t find the code source on the filesystem.')
            return False
        installable.update(self, bool(purge))
        if REQUEST is not None:
            return self.editCodeSource(
                manage_tabs_message='Source updated from the filesystem.')
        return True

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'manage_exportCodeSource')
    def manage_exportCodeSource(self, aszip=False, dump=False, REQUEST=None):
        """Export a code source to the filesystem.
        """
        if dump:
            installable = self._get_installable()
            if (installable is None or
                not os.path.isdir(installable._directory)):
                message="Couldn't find the code source on the filesystem."
            else:
                installable.export(self)
                message="Source dumped to the filesystem."
            if REQUEST is not None:
                return self.editCodeSource(manage_tabs_message=message)
            return None

        directory = tempfile.mkdtemp('-codesource-export')
        try:
            CodeSourceExportable().export(self, directory)
            result = io.BytesIO()
            archive = zipfile.ZipFile(result, 'w')
            for path, directories, filenames in os.walk(directory):
                root = path[len(directory):]
                if root:
                    root = os.path.join(self.getId(), root)
                else:
                    root = self.getId()
                for filename in filenames:
                    archive.write(os.path.join(path, filename),
                                  os.path.join(root, filename))
            archive.close()
        finally:
            shutil.rmtree(directory)
        if REQUEST is not None:
            REQUEST.RESPONSE.setHeader(
                'Content-Type',
                'application/zip')
            REQUEST.RESPONSE.setHeader(
                'Content-Disposition',
                'attachment;filename=%s.zip' % self.getId())
        return result.getvalue()

    security.declareProtected(
        SilvaPermissions.ViewManagementScreens, 'manage_editCodeSource')
    def manage_editCodeSource(
        self, title, script_id, data_encoding, description=None, location=None,
        cacheable=None, previewable=None, usable=None, script_layers=None):
        """ Edit a code source settings.
        """
        msg = u''

        if location is not None and location != self._fs_location:
            if location:
                installable = self._get_installable(location)
                if installable is None:
                    msg += "Invalid location for the code source " + \
                        "definition, not changed! "
                    return self.editCodeSource(manage_tabs_message=msg)
            self._fs_location = location
            msg += "Code source location changed. "

        if data_encoding != self._data_encoding:
            try:
                unicode('abcd', data_encoding, 'replace')
            except LookupError:
                # unknown encoding, return error message
                msg += "Unknown encoding %s, not changed! " % data_encoding
                return self.editCodeSource(manage_tabs_message=msg)
            self.set_data_encoding(data_encoding)
            msg += u'Data encoding changed. '

        if script_layers is not None:
            try:
                self.set_script_layers(script_layers)
            except ValueError as error:
                msg += "Error while setting script layers: %s" % error.args[0]
                return self.editCodeSource(manage_tabs_message=msg)

        title = unicode(title, self.management_page_charset)
        if title and title != self.title:
            self.title = title
            msg += "Title changed. "

        if script_id and script_id != self._script_id:
            self._script_id = script_id
            msg += "Script id changed. "

        # Assume description is in the encoding as specified
        # by "management_page_charset". Store it in unicode.
        if description is not None:
            description = unicode(description, self.management_page_charset)
            if description != self._description:
                self.set_description(description)
                msg += "Description changed. "

        if not bool(cacheable) is self.is_cacheable():
            self.set_cacheable(bool(cacheable))
            msg += "Cacheability setting changed. "

        if not bool(usable) is self.is_usable():
            self.set_usable(bool(usable))
            msg += "Usability setting changed. "

        if not bool(previewable) is self.is_previewable():
            self.set_previewable(bool(previewable))
            msg += "Previewable setting changed. "

        return self.editCodeSource(manage_tabs_message=msg)

InitializeClass(CodeSource)


manage_addCodeSourceForm = PageTemplateFile(
    "www/codeSourceAdd", globals(),
    __name__='manage_addCodeSourceForm')


def manage_addCodeSource(
    context, id, title=None, script_id=None, fs_location=None,REQUEST=None):
    """Add a CodeSource object
    """
    cs = CodeSource(id, script_id, fs_location)
    if title is not None:
        title = unicode(title, cs.management_page_charset)
        cs.set_title(title)
    context._setObject(id, cs)
    cs = context._getOb(id)
    cs.set_form(ZMIForm('form', 'Parameters form', unicode_mode=1))

    add_and_edit(context, id, REQUEST, screen='editCodeSource')
    return ''



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

# Helpers for cs_youtube ...

import urlparse

from AccessControl import ModuleSecurityInfo

module_security = ModuleSecurityInfo(
    'Products.SilvaExternalSources.codesources.youtube')


# correct urls have a path which start with '/v/' and youtube netloc
# urls have which have a path that starts with '/watch' or with '/embed'
# need to get formatted, invalid urls have a path different than that

module_security.declarePublic('validate_youtube_url')
def validate_youtube_url(value, REQUEST=None):
    """Validate that a url can be used as a youtube URL.

    >>> validate_youtube_url("http://www.youtube.com/watch?v=4T1BITS4Hz0&feature=g-all-u")
    True
    >>> validate_youtube_url("http://www.youtube.com/v/4T1BITS4Hz0")
    True
    >>> validate_youtube_url("http://youtube.com/v/4T1BITS4Hz0")
    True
    >>> validate_youtube_url("http://www.youtube.com/embed/fwUHKgVc2zc")
    True
    >>> validate_youtube_url("http://www.youtube.com/v/4T1BITS4Hz0?version=3&amp;hl=nl_NL")
    True
    >>> validate_youtube_url("http://youtu.be/Lh6QPg5BGsE")
    True
    >>> validate_youtube_url("http://youtube.be/v/Lh6QPg5BGsE")
    False
    >>> validate_youtube_url("http://youtube.com/embed?v=Lh6QPg5BGsE")
    False
    >>> validate_youtube_url("http://youtube.com/embed/videos/Lh6QPg5BGsE")
    False
    >>> validate_youtube_url("http://youtube.com/")
    False
    >>> validate_youtube_url("http://www.youtube.com/watch?foo=bar")
    False
    >>> validate_youtube_url("http://silvacms.org")
    False
    """
    return format_youtube_url(value) is not None


module_security.declarePublic('format_youtube_url')
def format_youtube_url(source_url):
    """Convert different format of youtube URLs to the one required by
    the flash player.

    >>> format_youtube_url("http://www.youtube.com/watch?v=4T1BITS4Hz0&feature=g-all-u")
    'http://www.youtube.com/v/4T1BITS4Hz0'
    >>> format_youtube_url("http://www.youtube.com/v/4T1BITS4Hz0")
    'http://www.youtube.com/v/4T1BITS4Hz0'
    >>> format_youtube_url("http://youtube.com/v/4T1BITS4Hz0")
    'http://www.youtube.com/v/4T1BITS4Hz0'
    >>> format_youtube_url("http://www.youtube.com/embed/fwUHKgVc2zc")
    'http://www.youtube.com/v/fwUHKgVc2zc'
    >>> format_youtube_url("http://www.youtube.com/v/4T1BITS4Hz0?version=3&amp;hl=nl_NL")
    'http://www.youtube.com/v/4T1BITS4Hz0'
    >>> format_youtube_url("http://youtu.be/Lh6QPg5BGsE")
    'http://www.youtube.com/v/Lh6QPg5BGsE'
    >>> format_youtube_url("http://youtube.be/v/Lh6QPg5BGsE")
    >>> format_youtube_url("http://youtube.com/embed?v=Lh6QPg5BGsE")
    >>> format_youtube_url("http://youtube.com/embed/videos/Lh6QPg5BGsE")
    >>> format_youtube_url("http://youtube.com/")
    >>> format_youtube_url("http://www.youtube.com/watch?foo=bar")
    """
    if source_url is None:
        return None

    parsed_url = urlparse.urlparse(source_url)
    parsed_query = urlparse.parse_qs(parsed_url.query)
    parsed_path = parsed_url.path.strip('/').split('/')

    if not len(parsed_path):
        return None

    def youtube_url(video_id):
        return urlparse.urlunsplit((
            parsed_url.scheme,
            'www.youtube.com',
            '/v/' + video_id,
            '',
            ''),)

    if parsed_url.netloc.endswith('youtube.com'):

        if parsed_path[0] == 'v' and len(parsed_path) == 2:
            return youtube_url(parsed_path[1])
        if parsed_path[0] == 'embed' and len(parsed_path) == 2:
            return youtube_url(parsed_path[1])

        if parsed_path[0] == 'watch':
            if 'v' in parsed_query:
                return youtube_url(parsed_query['v'][0])

    elif parsed_url.netloc.endswith('youtu.be'):

        if len(parsed_path) == 1:
            return youtube_url(parsed_path[0])

    return None







##parameters=REQUEST, model, version, vimeo_id, width, height

return """
<iframe src="http://player.vimeo.com/video/%(video_id)s" width="%(video_width)s" height="%(video_height)s" frameborder="0"></iframe>
""" % {"video_id": vimeo_id,
       "video_width": width,
       "video_height": height}
